@@ -27,13 +27,13 @@
 #include "stats/stats-registry.h"
 #include "messages.h"
 #include "apphook.h"
-#include "mainloop.h"
+#include "mainloop-threaded-worker.h"
 
 #include <iv_event.h>
 
 typedef struct _AFInterSource AFInterSource;
 
-static GStaticMutex internal_msg_lock = G_STATIC_MUTEX_INIT;
+static GMutex internal_msg_lock;
 static GQueue *internal_msg_queue;
 static AFInterSource *current_internal_source;
 static StatsCounterItem *internal_queue_length;
@@ -44,7 +44,7 @@ static struct timespec next_mark_target = { -1, 0 };
 /* as different sources from different threads can call afinter_postpone_mark,
    and we use the value in the init thread, we need to synchronize the value references
 */
-static GStaticMutex internal_mark_target_lock = G_STATIC_MUTEX_INIT;
+static GMutex internal_mark_target_lock;
 
 /*
  * This class is parallel to LogReader, e.g. it hangs on the
@@ -92,6 +92,7 @@ static GStaticMutex internal_mark_target_lock = G_STATIC_MUTEX_INIT;
 struct _AFInterSource
 {
   LogSource super;
+  MainLoopThreadedWorker thread;
   gint mark_freq;
   const AFInterSourceOptions *options;
   struct iv_event post;
@@ -113,14 +114,15 @@ afinter_source_post(gpointer s)
 
   while (log_source_free_to_send(&self->super))
     {
-      g_static_mutex_lock(&internal_msg_lock);
+      g_mutex_lock(&internal_msg_lock);
       msg = g_queue_pop_head(internal_msg_queue);
-      g_static_mutex_unlock(&internal_msg_lock);
+      g_mutex_unlock(&internal_msg_lock);
       if (!msg)
         break;
 
       stats_counter_dec(internal_queue_length);
       log_source_post(&self->super, msg);
+      main_loop_worker_invoke_batch_callbacks();
     }
   afinter_source_update_watches(self);
 }
@@ -128,74 +130,76 @@ afinter_source_post(gpointer s)
 static void afinter_source_start_watches(AFInterSource *self);
 static void afinter_source_stop_watches(AFInterSource *self);
 
-static void
-afinter_source_run(gpointer s)
+static gboolean
+afinter_source_thread_init(MainLoopThreadedWorker *s)
 {
-  AFInterSource *self = (AFInterSource *) s;
-
-  iv_init();
+  AFInterSource *self = (AFInterSource *) s->data;
 
   /* post event is used by other threads and can only be unregistered if
    * current_afinter_source is set to NULL in a thread safe manner */
   iv_event_register(&self->post);
   iv_event_register(&self->schedule_wakeup);
   iv_event_register(&self->exit);
+  return TRUE;
+}
 
-  g_static_mutex_lock(&internal_msg_lock);
+static void
+afinter_source_thread_deinit(MainLoopThreadedWorker *s)
+{
+  AFInterSource *self = (AFInterSource *) s->data;
+
+  iv_event_unregister(&self->exit);
+  iv_event_unregister(&self->post);
+  iv_event_unregister(&self->schedule_wakeup);
+}
+
+static void
+afinter_source_run(MainLoopThreadedWorker *s)
+{
+  AFInterSource *self = (AFInterSource *) s->data;
+
+  g_mutex_lock(&internal_msg_lock);
   self->free_to_send = TRUE;
-  g_static_mutex_unlock(&internal_msg_lock);
+  g_mutex_unlock(&internal_msg_lock);
 
   afinter_source_start_watches(self);
   afinter_source_update_watches(self);
 
   iv_main();
 
-  g_static_mutex_lock(&internal_msg_lock);
+  g_mutex_lock(&internal_msg_lock);
   current_internal_source = NULL;
-  g_static_mutex_unlock(&internal_msg_lock);
-
-  iv_event_unregister(&self->exit);
-  iv_event_unregister(&self->post);
-  iv_event_unregister(&self->schedule_wakeup);
+  g_mutex_unlock(&internal_msg_lock);
 
   afinter_source_stop_watches(self);
-
-  iv_deinit();
 }
 
 static void
-afinter_source_request_exit(gpointer s)
+afinter_source_thread_request_exit(MainLoopThreadedWorker *s)
 {
-  AFInterSource *self = (AFInterSource *) s;
+  AFInterSource *self = (AFInterSource *) s->data;
 
   iv_event_post(&self->exit);
 }
 
-
 static gboolean
-afinter_sd_start_thread(LogPipe *s)
+afinter_source_start_thread(LogSource *s)
 {
-  AFInterSourceDriver *self = (AFInterSourceDriver *) s;
-
-  self->worker_options.is_external_input = TRUE;
-
-  main_loop_create_worker_thread((WorkerThreadFunc) afinter_source_run,
-                                 (WorkerExitNotificationFunc) afinter_source_request_exit,
-                                 self->source, &self->worker_options);
+  AFInterSource *self = (AFInterSource *) s;
 
+  main_loop_threaded_worker_start(&self->thread);
   return TRUE;
 }
 
-
 static void
 afinter_source_mark(gpointer s)
 {
   AFInterSource *self = (AFInterSource *) s;
   struct timespec nmt;
 
-  g_static_mutex_lock(&internal_mark_target_lock);
+  g_mutex_lock(&internal_mark_target_lock);
   nmt = next_mark_target;
-  g_static_mutex_unlock(&internal_mark_target_lock);
+  g_mutex_unlock(&internal_mark_target_lock);
 
   if (nmt.tv_sec <= self->mark_timer.expires.tv_sec)
     {
@@ -278,9 +282,9 @@ afinter_source_update_watches(AFInterSource *self)
     {
       /* ok, we go to sleep now. let's disable the post event.
        * Messages get accumulated into internal_msg_queue.  */
-      g_static_mutex_lock(&internal_msg_lock);
+      g_mutex_lock(&internal_msg_lock);
       self->free_to_send = FALSE;
-      g_static_mutex_unlock(&internal_msg_lock);
+      g_mutex_unlock(&internal_msg_lock);
 
       /* Possible race:
        *
@@ -320,11 +324,11 @@ afinter_source_update_watches(AFInterSource *self)
        * be triggered.
        */
 
-      g_static_mutex_lock(&internal_msg_lock);
+      g_mutex_lock(&internal_msg_lock);
       if (internal_msg_queue && g_queue_get_length(internal_msg_queue) > 0)
         iv_task_register(&self->restart_task);
       self->free_to_send = TRUE;
-      g_static_mutex_unlock(&internal_msg_lock);
+      g_mutex_unlock(&internal_msg_lock);
     }
 }
 
@@ -341,9 +345,9 @@ afinter_source_init(LogPipe *s)
   afinter_postpone_mark(self->mark_freq);
   self->mark_timer.expires = next_mark_target;
 
-  g_static_mutex_lock(&internal_msg_lock);
+  g_mutex_lock(&internal_msg_lock);
   current_internal_source = self;
-  g_static_mutex_unlock(&internal_msg_lock);
+  g_mutex_unlock(&internal_msg_lock);
 
   return TRUE;
 }
@@ -353,13 +357,22 @@ afinter_source_deinit(LogPipe *s)
 {
   AFInterSource *self = (AFInterSource *) s;
 
-  g_static_mutex_lock(&internal_msg_lock);
+  g_mutex_lock(&internal_msg_lock);
   current_internal_source = NULL;
-  g_static_mutex_unlock(&internal_msg_lock);
+  g_mutex_unlock(&internal_msg_lock);
 
   return log_source_deinit(&self->super.super);
 }
 
+static void
+afinter_source_free(LogPipe *s)
+{
+  AFInterSource *self = (AFInterSource *) s;
+
+  main_loop_threaded_worker_clear(&self->thread);
+  log_source_free(s);
+}
+
 static LogSource *
 afinter_source_new(AFInterSourceDriver *owner, AFInterSourceOptions *options)
 {
@@ -368,10 +381,17 @@ afinter_source_new(AFInterSourceDriver *owner, AFInterSourceOptions *options)
   log_source_init_instance(&self->super, owner->super.super.super.cfg);
   log_source_set_options(&self->super, &options->super, owner->super.super.id, NULL, FALSE,
                          owner->super.super.super.expr_node);
+  main_loop_threaded_worker_init(&self->thread, MLW_THREADED_INPUT_WORKER, self);
+  self->thread.thread_init = afinter_source_thread_init;
+  self->thread.thread_deinit = afinter_source_thread_deinit;
+  self->thread.run = afinter_source_run;
+  self->thread.request_exit = afinter_source_thread_request_exit;
+
   afinter_source_init_watches(self);
   self->super.super.init = afinter_source_init;
   self->super.super.deinit = afinter_source_deinit;
   self->super.wakeup = afinter_source_wakeup;
+  self->super.super.free_fn = afinter_source_free;
 
   self->options = options;
 
@@ -386,6 +406,13 @@ afinter_source_options_defaults(AFInterSourceOptions *options)
   options->queue_capacity = 10000;
 }
 
+static gboolean
+afinter_sd_pre_config_init(LogPipe *s)
+{
+  main_loop_worker_allocate_thread_space(1);
+  return TRUE;
+}
+
 static gboolean
 afinter_sd_init(LogPipe *s)
 {
@@ -423,6 +450,14 @@ afinter_sd_init(LogPipe *s)
   return TRUE;
 }
 
+static gboolean
+afinter_sd_post_config_init(LogPipe *s)
+{
+  AFInterSourceDriver *self = (AFInterSourceDriver *) s;
+
+  return afinter_source_start_thread(self->source);
+}
+
 static gboolean
 afinter_sd_deinit(LogPipe *s)
 {
@@ -458,10 +493,11 @@ afinter_sd_new(GlobalConfig *cfg)
   AFInterSourceDriver *self = g_new0(AFInterSourceDriver, 1);
 
   log_src_driver_init_instance((LogSrcDriver *)&self->super, cfg);
+  self->super.super.super.pre_config_init = afinter_sd_pre_config_init;
   self->super.super.super.init = afinter_sd_init;
   self->super.super.super.deinit = afinter_sd_deinit;
   self->super.super.super.free_fn = afinter_sd_free;
-  self->super.super.super.on_config_inited = afinter_sd_start_thread;
+  self->super.super.super.post_config_init = afinter_sd_post_config_init;
 
   afinter_source_options_defaults(&self->source_options);
 
@@ -478,16 +514,16 @@ afinter_postpone_mark(gint mark_freq)
   if (mark_freq > 0)
     {
       iv_validate_now();
-      g_static_mutex_lock(&internal_mark_target_lock);
+      g_mutex_lock(&internal_mark_target_lock);
       next_mark_target = iv_now;
       next_mark_target.tv_sec += mark_freq;
-      g_static_mutex_unlock(&internal_mark_target_lock);
+      g_mutex_unlock(&internal_mark_target_lock);
     }
   else
     {
-      g_static_mutex_lock(&internal_mark_target_lock);
+      g_mutex_lock(&internal_mark_target_lock);
       next_mark_target.tv_sec = -1;
-      g_static_mutex_unlock(&internal_mark_target_lock);
+      g_mutex_unlock(&internal_mark_target_lock);
     }
 }
 
@@ -511,7 +547,7 @@ _register_obsolete_stats_alias(StatsCounterItem *internal_queued_ctr)
 {
   stats_lock();
   StatsClusterKey sc_key;
-  stats_cluster_logpipe_key_set(&sc_key, SCS_GLOBAL, "internal_queue_length", NULL);
+  stats_cluster_logpipe_key_legacy_set(&sc_key, SCS_GLOBAL, "internal_queue_length", NULL);
   stats_register_alias_counter(0, &sc_key, SC_TYPE_PROCESSED, internal_queued_ctr);
   stats_unlock();
 }
@@ -521,7 +557,7 @@ _unregister_obsolete_stats_alias(StatsCounterItem *internal_queued_ctr)
 {
   stats_lock();
   StatsClusterKey sc_key;
-  stats_cluster_logpipe_key_set(&sc_key, SCS_GLOBAL, "internal_queue_length", NULL);
+  stats_cluster_logpipe_key_legacy_set(&sc_key, SCS_GLOBAL, "internal_queue_length", NULL);
   stats_unregister_alias_counter(&sc_key, SC_TYPE_PROCESSED, internal_queued_ctr);
   stats_unlock();
 }
@@ -529,7 +565,7 @@ _unregister_obsolete_stats_alias(StatsCounterItem *internal_queued_ctr)
 void
 afinter_message_posted(LogMessage *msg)
 {
-  g_static_mutex_lock(&internal_msg_lock);
+  g_mutex_lock(&internal_msg_lock);
   if (!current_internal_source)
     {
       if (internal_msg_queue)
@@ -546,7 +582,8 @@ afinter_message_posted(LogMessage *msg)
 
       stats_lock();
       StatsClusterKey sc_key;
-      stats_cluster_logpipe_key_set(&sc_key, SCS_GLOBAL, "internal_source", NULL );
+      stats_cluster_logpipe_key_set(&sc_key, "internal_source", NULL, 0);
+      stats_cluster_logpipe_key_add_legacy_alias(&sc_key, SCS_GLOBAL, "internal_source", NULL );
       stats_register_counter(0, &sc_key, SC_TYPE_QUEUED, &internal_queue_length);
       stats_register_counter(0, &sc_key, SC_TYPE_DROPPED, &internal_queue_dropped);
       stats_unlock();
@@ -567,7 +604,7 @@ afinter_message_posted(LogMessage *msg)
   if (current_internal_source->free_to_send)
     iv_event_post(&current_internal_source->post);
 exit:
-  g_static_mutex_unlock(&internal_msg_lock);
+  g_mutex_unlock(&internal_msg_lock);
 }
 
 static void
@@ -591,7 +628,8 @@ afinter_global_deinit(void)
 
       stats_lock();
       StatsClusterKey sc_key;
-      stats_cluster_logpipe_key_set(&sc_key, SCS_GLOBAL, "internal_source", NULL );
+      stats_cluster_logpipe_key_set(&sc_key, "internal_source", NULL, 0);
+      stats_cluster_logpipe_key_add_legacy_alias(&sc_key, SCS_GLOBAL, "internal_source", NULL );
       stats_unregister_counter(&sc_key, SC_TYPE_QUEUED, &internal_queue_length);
       stats_unregister_counter(&sc_key, SC_TYPE_DROPPED, &internal_queue_dropped);
       stats_unlock();
