@@ -45,6 +45,7 @@
 #include "transport/transport-factory-id.h"
 #include "timeutils/timeutils.h"
 #include "msg-stats.h"
+#include "timeutils/cache.h"
 
 #include <iv.h>
 #include <iv_work.h>
@@ -57,7 +58,15 @@ typedef struct _ApplicationHookEntry
   gpointer user_data;
 } ApplicationHookEntry;
 
+typedef struct _ApplicationThreadHookEntry
+{
+  ApplicationThreadHookFunc func;
+  gpointer user_data;
+} ApplicationThreadHookEntry;
+
 static GList *application_hooks = NULL;
+static GList *application_thread_init_hooks = NULL;
+static GList *application_thread_deinit_hooks = NULL;
 static gint current_state = AH_STARTUP;
 
 gboolean
@@ -96,6 +105,48 @@ register_application_hook(gint type, ApplicationHookFunc func, gpointer user_dat
     }
 }
 
+void
+register_application_thread_init_hook(ApplicationThreadHookFunc func, gpointer user_data)
+{
+  ApplicationThreadHookEntry *entry = g_new0(ApplicationThreadHookEntry, 1);
+
+  entry->func = func;
+  entry->user_data = user_data;
+
+  application_thread_init_hooks = g_list_prepend(application_thread_init_hooks, entry);
+}
+
+void
+register_application_thread_deinit_hook(ApplicationThreadHookFunc func, gpointer user_data)
+{
+  ApplicationThreadHookEntry *entry = g_new0(ApplicationThreadHookEntry, 1);
+
+  entry->func = func;
+  entry->user_data = user_data;
+
+  application_thread_deinit_hooks = g_list_prepend(application_thread_deinit_hooks, entry);
+}
+
+static void
+run_application_thread_init_hooks(void)
+{
+  for (GList *elem = application_thread_init_hooks; elem; elem = elem->next)
+    {
+      ApplicationThreadHookEntry *hook = (ApplicationThreadHookEntry *) elem->data;
+      hook->func(hook->user_data);
+    }
+}
+
+static void
+run_application_thread_deinit_hooks(void)
+{
+  for (GList *elem = application_thread_deinit_hooks; elem; elem = elem->next)
+    {
+      ApplicationThreadHookEntry *hook = (ApplicationThreadHookEntry *) elem->data;
+      hook->func(hook->user_data);
+    }
+}
+
 static void
 _update_current_state(gint type)
 {
@@ -171,7 +222,6 @@ app_startup(void)
   afinter_global_init();
   child_manager_init();
   alarm_init();
-  g_thread_init(NULL);
   main_loop_thread_resource_init();
   stats_init();
   tzset();
@@ -228,6 +278,8 @@ app_shutdown(void)
   child_manager_deinit();
   g_list_foreach(application_hooks, (GFunc) g_free, NULL);
   g_list_free(application_hooks);
+  g_list_free_full(application_thread_init_hooks, g_free);
+  g_list_free_full(application_thread_deinit_hooks, g_free);
   dns_caching_thread_deinit();
   dns_caching_global_deinit();
   hostname_global_deinit();
@@ -249,6 +301,18 @@ app_shutdown(void)
    */
 }
 
+void
+app_config_pre_pre_init(void)
+{
+  run_application_hook(AH_CONFIG_PRE_PRE_INIT);
+}
+
+void
+app_config_pre_init(void)
+{
+  run_application_hook(AH_CONFIG_PRE_INIT);
+}
+
 void
 app_config_stopped(void)
 {
@@ -273,12 +337,15 @@ app_thread_start(void)
   scratch_buffers_allocator_init();
   dns_caching_thread_init();
   main_loop_call_thread_init();
+  run_application_thread_init_hooks();
 }
 
 void
 app_thread_stop(void)
 {
+  run_application_thread_deinit_hooks();
   main_loop_call_thread_deinit();
   dns_caching_thread_deinit();
   scratch_buffers_allocator_deinit();
+  timeutils_cache_deinit();
 }
