@@ -26,6 +26,7 @@
 #include "logqueue-fifo.h"
 #include "afinter.h"
 #include "cfg-tree.h"
+#include "messages.h"
 
 #include <string.h>
 
@@ -38,21 +39,45 @@ log_driver_plugin_free_method(LogDriverPlugin *self)
 }
 
 void
-log_driver_plugin_init_instance(LogDriverPlugin *self)
+log_driver_plugin_init_instance(LogDriverPlugin *self, const gchar *name)
 {
+  self->name = name;
   self->free_fn = log_driver_plugin_free_method;
 }
 
 /* LogDriver */
 
-void
+gboolean
 log_driver_add_plugin(LogDriver *self, LogDriverPlugin *plugin)
 {
+  g_assert(plugin->name);
+  if (log_driver_lookup_plugin(self, plugin->name))
+    {
+      msg_error("Another instance of this plugin is registered in this driver, unable to register plugin again",
+                evt_tag_str("driver", self->id),
+                evt_tag_str("plugin", plugin->name));
+      return FALSE;
+
+    }
   self->plugins = g_list_append(self->plugins, plugin);
+  return TRUE;
 }
 
-gboolean
-log_driver_init_method(LogPipe *s)
+LogDriverPlugin *
+log_driver_lookup_plugin(LogDriver *self, const gchar *plugin_name)
+{
+  for (GList *l = self->plugins; l; l = l->next)
+    {
+      LogDriverPlugin *plugin = (LogDriverPlugin *) l->data;
+
+      if (strcmp(plugin->name, plugin_name) == 0)
+        return plugin;
+    }
+  return NULL;
+}
+
+static gboolean
+log_driver_pre_init_method(LogPipe *s)
 {
   LogDriver *self = (LogDriver *) s;
   gboolean success = TRUE;
@@ -66,18 +91,28 @@ log_driver_init_method(LogPipe *s)
   return success;
 }
 
-gboolean
-log_driver_deinit_method(LogPipe *s)
+static void
+log_driver_post_deinit_method(LogPipe *s)
 {
   LogDriver *self = (LogDriver *) s;
-  gboolean success = TRUE;
   GList *l;
 
   for (l = self->plugins; l; l = l->next)
     {
       log_driver_plugin_detach((LogDriverPlugin *) l->data, self);
     }
-  return success;
+}
+
+gboolean
+log_driver_init_method(LogPipe *s)
+{
+  return TRUE;
+}
+
+gboolean
+log_driver_deinit_method(LogPipe *s)
+{
+  return TRUE;
 }
 
 /* NOTE: intentionally static, as only cDriver or LogDestDriver will derive from LogDriver */
@@ -107,21 +142,21 @@ static void
 log_driver_init_instance(LogDriver *self, GlobalConfig *cfg)
 {
   log_pipe_init_instance(&self->super, cfg);
+  self->super.flags |= PIF_CONFIG_RELATED;
   self->super.free_fn = log_driver_free;
+  self->super.pre_init = log_driver_pre_init_method;
   self->super.init = log_driver_init_method;
   self->super.deinit = log_driver_deinit_method;
+  self->super.post_deinit = log_driver_post_deinit_method;
 }
 
 /* LogSrcDriver */
 
-gboolean
-log_src_driver_init_method(LogPipe *s)
+static gboolean
+log_src_driver_pre_init_method(LogPipe *s)
 {
   LogSrcDriver *self = (LogSrcDriver *) s;
-  GlobalConfig *cfg = log_pipe_get_config(s);
-
-  if (!log_driver_init_method(s))
-    return FALSE;
+  GlobalConfig *cfg = log_pipe_get_config(&self->super.super);
 
   if (!self->super.group)
     {
@@ -129,11 +164,24 @@ log_src_driver_init_method(LogPipe *s)
       self->group_len = strlen(self->super.group);
       self->super.id = cfg_tree_get_child_id(&cfg->tree, ENC_SOURCE, s->expr_node);
     }
+  return log_driver_pre_init_method(s);
+}
+
+gboolean
+log_src_driver_init_method(LogPipe *s)
+{
+  LogSrcDriver *self = (LogSrcDriver *) s;
+
+  if (!log_driver_init_method(s))
+    return FALSE;
 
   stats_lock();
-  stats_register_counter(0, SCS_SOURCE | SCS_GROUP, self->super.group, NULL, SC_TYPE_PROCESSED,
+  StatsClusterKey sc_key;
+  stats_cluster_logpipe_key_legacy_set(&sc_key, SCS_SOURCE | SCS_GROUP, self->super.group, NULL );
+  stats_register_counter(0, &sc_key, SC_TYPE_PROCESSED,
                          &self->super.processed_group_messages);
-  stats_register_counter(0, SCS_CENTER, NULL, "received", SC_TYPE_PROCESSED, &self->received_global_messages);
+  stats_cluster_logpipe_key_legacy_set(&sc_key,  SCS_CENTER, NULL, "received" );
+  stats_register_counter(0, &sc_key, SC_TYPE_PROCESSED, &self->received_global_messages);
   stats_unlock();
 
   return TRUE;
@@ -148,15 +196,18 @@ log_src_driver_deinit_method(LogPipe *s)
     return FALSE;
 
   stats_lock();
-  stats_unregister_counter(SCS_SOURCE | SCS_GROUP, self->super.group, NULL, SC_TYPE_PROCESSED,
+  StatsClusterKey sc_key;
+  stats_cluster_logpipe_key_legacy_set(&sc_key, SCS_SOURCE | SCS_GROUP, self->super.group, NULL );
+  stats_unregister_counter(&sc_key, SC_TYPE_PROCESSED,
                            &self->super.processed_group_messages);
-  stats_unregister_counter(SCS_CENTER, NULL, "received", SC_TYPE_PROCESSED, &self->received_global_messages);
+  stats_cluster_logpipe_key_legacy_set(&sc_key, SCS_CENTER, NULL, "received" );
+  stats_unregister_counter(&sc_key, SC_TYPE_PROCESSED, &self->received_global_messages);
   stats_unlock();
   return TRUE;
 }
 
 void
-log_src_driver_queue_method(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options, gpointer user_data)
+log_src_driver_queue_method(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options)
 {
   LogSrcDriver *self = (LogSrcDriver *) s;
   GlobalConfig *cfg = log_pipe_get_config(s);
@@ -176,6 +227,7 @@ void
 log_src_driver_init_instance(LogSrcDriver *self, GlobalConfig *cfg)
 {
   log_driver_init_instance(&self->super, cfg);
+  self->super.super.pre_init = log_src_driver_pre_init_method;
   self->super.super.init = log_src_driver_init_method;
   self->super.super.deinit = log_src_driver_deinit_method;
   self->super.super.queue = log_src_driver_queue_method;
@@ -190,22 +242,46 @@ log_src_driver_free(LogPipe *s)
 
 /* LogDestDriver */
 
+static LogQueue *
+_create_memory_queue(LogDestDriver *self, const gchar *persist_name)
+{
+  GlobalConfig *cfg = log_pipe_get_config(&self->super.super);
+
+  gint log_fifo_size = self->log_fifo_size < 0 ? cfg->log_fifo_size : self->log_fifo_size;
+
+  if (cfg_is_config_version_older(cfg, VERSION_VALUE_3_22))
+    {
+      msg_warning_once("WARNING: log-fifo-size() works differently starting with " VERSION_3_22 " to avoid dropping "
+                       "flow-controlled messages when log-fifo-size() is misconfigured. From now on, log-fifo-size() "
+                       "only affects messages that are not flow-controlled. (Flow-controlled log paths have the "
+                       "flags(flow-control) option set.) To enable the new behaviour, update the @version string in "
+                       "your configuration and consider lowering the value of log-fifo-size().");
+
+      return log_queue_fifo_legacy_new(log_fifo_size, persist_name);
+    }
+
+  return log_queue_fifo_new(log_fifo_size, persist_name);
+}
+
 /* returns a reference */
 static LogQueue *
-log_dest_driver_acquire_queue_method(LogDestDriver *self, const gchar *persist_name,
-                                     gpointer user_data)
+log_dest_driver_acquire_memory_queue(LogDestDriver *self, const gchar *persist_name)
 {
   GlobalConfig *cfg = log_pipe_get_config(&self->super.super);
   LogQueue *queue = NULL;
 
-  g_assert(user_data == NULL);
-
   if (persist_name)
     queue = cfg_persist_config_fetch(cfg, persist_name);
 
+  if (queue && !log_queue_has_type(queue, log_queue_fifo_get_type()))
+    {
+      log_queue_unref(queue);
+      queue = NULL;
+    }
+
   if (!queue)
     {
-      queue = log_queue_fifo_new(self->log_fifo_size < 0 ? cfg->log_fifo_size : self->log_fifo_size, persist_name);
+      queue = _create_memory_queue(self, persist_name);
       log_queue_set_throttle(queue, self->throttle);
     }
   return queue;
@@ -213,7 +289,7 @@ log_dest_driver_acquire_queue_method(LogDestDriver *self, const gchar *persist_n
 
 /* consumes the reference in @q */
 static void
-log_dest_driver_release_queue_method(LogDestDriver *self, LogQueue *q, gpointer user_data)
+log_dest_driver_release_queue_method(LogDestDriver *self, LogQueue *q)
 {
   GlobalConfig *cfg = log_pipe_get_config(&self->super.super);
 
@@ -229,7 +305,7 @@ log_dest_driver_release_queue_method(LogDestDriver *self, LogQueue *q, gpointer
 }
 
 void
-log_dest_driver_queue_method(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options, gpointer user_data)
+log_dest_driver_queue_method(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options)
 {
   LogDestDriver *self = (LogDestDriver *) s;
 
@@ -238,25 +314,35 @@ log_dest_driver_queue_method(LogPipe *s, LogMessage *msg, const LogPathOptions *
   log_pipe_forward_msg(s, msg, path_options);
 }
 
-gboolean
-log_dest_driver_init_method(LogPipe *s)
+static gboolean
+log_dest_driver_pre_init_method(LogPipe *s)
 {
   LogDestDriver *self = (LogDestDriver *) s;
-  GlobalConfig *cfg = log_pipe_get_config(s);
-
-  if (!log_driver_init_method(s))
-    return FALSE;
+  GlobalConfig *cfg = log_pipe_get_config(&self->super.super);
 
   if (!self->super.group)
     {
       self->super.group = cfg_tree_get_rule_name(&cfg->tree, ENC_DESTINATION, s->expr_node);
       self->super.id = cfg_tree_get_child_id(&cfg->tree, ENC_DESTINATION, s->expr_node);
     }
+  return log_driver_pre_init_method(s);
+}
+
+gboolean
+log_dest_driver_init_method(LogPipe *s)
+{
+  LogDestDriver *self = (LogDestDriver *) s;
+
+  if (!log_driver_init_method(s))
+    return FALSE;
 
   stats_lock();
-  stats_register_counter(0, SCS_DESTINATION | SCS_GROUP, self->super.group, NULL, SC_TYPE_PROCESSED,
+  StatsClusterKey sc_key;
+  stats_cluster_logpipe_key_legacy_set(&sc_key, SCS_DESTINATION | SCS_GROUP, self->super.group, NULL );
+  stats_register_counter(0, &sc_key, SC_TYPE_PROCESSED,
                          &self->super.processed_group_messages);
-  stats_register_counter(0, SCS_CENTER, NULL, "queued", SC_TYPE_PROCESSED, &self->queued_global_messages);
+  stats_cluster_logpipe_key_legacy_set(&sc_key, SCS_CENTER, NULL, "queued" );
+  stats_register_counter(0, &sc_key, SC_TYPE_PROCESSED, &self->queued_global_messages);
   stats_unlock();
 
   return TRUE;
@@ -282,9 +368,12 @@ log_dest_driver_deinit_method(LogPipe *s)
   g_assert(self->queues == NULL);
 
   stats_lock();
-  stats_unregister_counter(SCS_DESTINATION | SCS_GROUP, self->super.group, NULL, SC_TYPE_PROCESSED,
+  StatsClusterKey sc_key;
+  stats_cluster_logpipe_key_legacy_set(&sc_key, SCS_DESTINATION | SCS_GROUP, self->super.group, NULL );
+  stats_unregister_counter(&sc_key, SC_TYPE_PROCESSED,
                            &self->super.processed_group_messages);
-  stats_unregister_counter(SCS_CENTER, NULL, "queued", SC_TYPE_PROCESSED, &self->queued_global_messages);
+  stats_cluster_logpipe_key_legacy_set(&sc_key, SCS_CENTER, NULL, "queued" );
+  stats_unregister_counter(&sc_key, SC_TYPE_PROCESSED, &self->queued_global_messages);
   stats_unlock();
 
   if (!log_driver_deinit_method(s))
@@ -296,10 +385,11 @@ void
 log_dest_driver_init_instance(LogDestDriver *self, GlobalConfig *cfg)
 {
   log_driver_init_instance(&self->super, cfg);
+  self->super.super.pre_init = log_dest_driver_pre_init_method;
   self->super.super.init = log_dest_driver_init_method;
   self->super.super.deinit = log_dest_driver_deinit_method;
   self->super.super.queue = log_dest_driver_queue_method;
-  self->acquire_queue = log_dest_driver_acquire_queue_method;
+  self->acquire_queue = log_dest_driver_acquire_memory_queue;
   self->release_queue = log_dest_driver_release_queue_method;
   self->log_fifo_size = -1;
   self->throttle = 0;
