@@ -1497,7 +1497,7 @@ log_msg_clone_cow(LogMessage *msg, const LogPathOptions *path_options)
       self->ack_func = log_msg_clone_ack;
     }
 
-  self->flags &= ~LF_STATE_MASK;
+  self->flags &= ~(LF_STATE_MASK - LF_STATE_CLONED_MASK);
 
   if (self->num_tags == 0)
     self->flags |= LF_STATE_OWN_TAGS;
@@ -2034,16 +2034,17 @@ log_msg_register_stats(void)
 {
   stats_lock();
   StatsClusterKey sc_key;
-  stats_cluster_logpipe_key_set(&sc_key, SCS_GLOBAL, "msg_clones", NULL );
+  stats_cluster_logpipe_key_legacy_set(&sc_key, SCS_GLOBAL, "msg_clones", NULL );
   stats_register_counter(0, &sc_key, SC_TYPE_PROCESSED, &count_msg_clones);
 
-  stats_cluster_logpipe_key_set(&sc_key, SCS_GLOBAL, "payload_reallocs", NULL );
+  stats_cluster_logpipe_key_legacy_set(&sc_key, SCS_GLOBAL, "payload_reallocs", NULL );
   stats_register_counter(0, &sc_key, SC_TYPE_PROCESSED, &count_payload_reallocs);
 
-  stats_cluster_logpipe_key_set(&sc_key, SCS_GLOBAL, "sdata_updates", NULL );
+  stats_cluster_logpipe_key_legacy_set(&sc_key, SCS_GLOBAL, "sdata_updates", NULL );
   stats_register_counter(0, &sc_key, SC_TYPE_PROCESSED, &count_sdata_updates);
 
-  stats_cluster_single_key_set(&sc_key, SCS_GLOBAL, "msg_allocated_bytes", NULL);
+  stats_cluster_single_key_set(&sc_key, "events_allocated_bytes", NULL, 0);
+  stats_cluster_single_key_add_legacy_alias(&sc_key, SCS_GLOBAL, "msg_allocated_bytes", NULL);
   stats_register_counter(1, &sc_key, SC_TYPE_SINGLE_VALUE, &count_allocated_bytes);
   stats_unlock();
 }
