@@ -63,13 +63,14 @@ log_tags_get_by_name(const gchar *name)
 
      In both cases the return value is 0.
    */
-  guint id;
 
   g_assert(log_tags_hash != NULL);
 
   g_mutex_lock(&log_tags_lock);
 
-  id = GPOINTER_TO_UINT(g_hash_table_lookup(log_tags_hash, name)) - 1;
+  gpointer key = g_hash_table_lookup(log_tags_hash, name);
+  guint id = GPOINTER_TO_UINT(key) - 1;
+
   if (id == 0xffffffff)
     {
       if (log_tags_num < LOG_TAGS_MAX - 1)
@@ -90,7 +91,9 @@ log_tags_get_by_name(const gchar *name)
 
           stats_lock();
           StatsClusterKey sc_key;
-          stats_cluster_logpipe_key_set(&sc_key, SCS_TAG, name, NULL );
+          StatsClusterLabel labels[] = { stats_cluster_label("id", name) };
+          stats_cluster_logpipe_key_set(&sc_key, "tagged_events_total", labels, G_N_ELEMENTS(labels));
+          stats_cluster_logpipe_key_add_legacy_alias(&sc_key, SCS_TAG, name, NULL );
           stats_register_counter(3, &sc_key, SC_TYPE_PROCESSED, &log_tags_list[id].counter);
           stats_unlock();
 
@@ -177,7 +180,9 @@ log_tags_reinit_stats(void)
     {
       const gchar *name = log_tags_list[id].name;
       StatsClusterKey sc_key;
-      stats_cluster_logpipe_key_set(&sc_key, SCS_TAG, name, NULL );
+      StatsClusterLabel labels[] = { stats_cluster_label("id", name) };
+      stats_cluster_logpipe_key_set(&sc_key, "tagged_events_total", labels, G_N_ELEMENTS(labels));
+      stats_cluster_logpipe_key_add_legacy_alias(&sc_key, SCS_TAG, name, NULL );
 
       if (stats_check_level(3))
         stats_register_counter(3, &sc_key, SC_TYPE_PROCESSED, &log_tags_list[id].counter);
@@ -219,7 +224,9 @@ log_tags_global_deinit(void)
   StatsClusterKey sc_key;
   for (i = 0; i < log_tags_num; i++)
     {
-      stats_cluster_logpipe_key_set(&sc_key, SCS_TAG, log_tags_list[i].name, NULL );
+      StatsClusterLabel labels[] = { stats_cluster_label("id", log_tags_list[i].name) };
+      stats_cluster_logpipe_key_set(&sc_key, "tagged_events_total", labels, G_N_ELEMENTS(labels));
+      stats_cluster_logpipe_key_add_legacy_alias(&sc_key, SCS_TAG, log_tags_list[i].name, NULL );
       stats_unregister_counter(&sc_key, SC_TYPE_PROCESSED, &log_tags_list[i].counter);
       g_free(log_tags_list[i].name);
     }
