@@ -451,7 +451,7 @@ log_reader_handle_line(LogReader *self, const guchar *line, gint length, LogTran
 
   m = msg_format_construct_message(&self->options->parse_options, line, length);
   msg_debug("Incoming log entry",
-            evt_tag_printf("input", "%.*s", length, line),
+            evt_tag_mem("input", line, length),
             evt_tag_msg_reference(m));
 
   msg_format_parse_into(&self->options->parse_options, m, line, length);
@@ -585,22 +585,26 @@ static void
 _register_aggregated_stats(LogReader *self)
 {
   StatsClusterKey sc_key_eps_input;
-  stats_cluster_logpipe_key_set(&sc_key_eps_input, self->super.options->stats_source | SCS_SOURCE, self->super.stats_id,
-                                self->super.stats_instance);
+  stats_cluster_logpipe_key_legacy_set(&sc_key_eps_input, self->super.options->stats_source | SCS_SOURCE,
+                                       self->super.stats_id,
+                                       self->super.stats_instance);
 
   stats_aggregator_lock();
   StatsClusterKey sc_key;
 
-  stats_cluster_single_key_set_with_name(&sc_key, self->super.options->stats_source | SCS_SOURCE, self->super.stats_id,
-                                         self->super.stats_instance, "msg_size_max");
+  stats_cluster_single_key_legacy_set_with_name(&sc_key, self->super.options->stats_source | SCS_SOURCE,
+                                                self->super.stats_id,
+                                                self->super.stats_instance, "msg_size_max");
   stats_register_aggregator_maximum(self->super.options->stats_level, &sc_key, &self->max_message_size);
 
-  stats_cluster_single_key_set_with_name(&sc_key, self->super.options->stats_source | SCS_SOURCE, self->super.stats_id,
-                                         self->super.stats_instance, "msg_size_avg");
+  stats_cluster_single_key_legacy_set_with_name(&sc_key, self->super.options->stats_source | SCS_SOURCE,
+                                                self->super.stats_id,
+                                                self->super.stats_instance, "msg_size_avg");
   stats_register_aggregator_average(self->super.options->stats_level, &sc_key, &self->average_messages_size);
 
-  stats_cluster_single_key_set_with_name(&sc_key, self->super.options->stats_source | SCS_SOURCE, self->super.stats_id,
-                                         self->super.stats_instance, "eps");
+  stats_cluster_single_key_legacy_set_with_name(&sc_key, self->super.options->stats_source | SCS_SOURCE,
+                                                self->super.stats_id,
+                                                self->super.stats_instance, "eps");
   stats_register_aggregator_cps(self->super.options->stats_level, &sc_key, &sc_key_eps_input, SC_TYPE_PROCESSED,
                                 &self->CPS);
 
