@@ -219,7 +219,7 @@ log_source_dynamic_window_update_statistics(LogSource *self)
 static void
 _reclaim_dynamic_window(LogSource *self, gsize window_size)
 {
-  g_assert(self->full_window_size - window_size >= self->options->init_window_size);
+  g_assert(self->full_window_size - window_size >= self->initial_window_size);
   atomic_gssize_set(&self->window_size_to_be_reclaimed, window_size);
 }
 
@@ -228,7 +228,7 @@ _release_dynamic_window(LogSource *self)
 {
   g_assert(self->ack_tracker == NULL);
 
-  gsize dynamic_part = self->full_window_size - self->options->init_window_size;
+  gsize dynamic_part = self->full_window_size - self->initial_window_size;
   msg_trace("Releasing dynamic part of the window", evt_tag_int("dynamic_window_to_be_released", dynamic_part),
             log_pipe_location_tag(&self->super));
 
@@ -334,7 +334,7 @@ _reclaim_window_instead_of_rebalance(LogSource *self)
 static void
 _dynamic_window_rebalance(LogSource *self)
 {
-  gsize current_dynamic_win = self->full_window_size - self->options->init_window_size;
+  gsize current_dynamic_win = self->full_window_size - self->initial_window_size;
   gboolean have_to_increase = current_dynamic_win < self->dynamic_window.pool->balanced_window;
   gboolean have_to_decrease = current_dynamic_win > self->dynamic_window.pool->balanced_window;
 
@@ -343,7 +343,7 @@ _dynamic_window_rebalance(LogSource *self)
             evt_tag_printf("connection", "%p", self),
             evt_tag_int("full_window", self->full_window_size),
             evt_tag_int("dynamic_win", current_dynamic_win),
-            evt_tag_int("static_window", self->options->init_window_size),
+            evt_tag_int("static_window", self->initial_window_size),
             evt_tag_int("balanced_window", self->dynamic_window.pool->balanced_window),
             evt_tag_int("avg_free", dynamic_window_stat_get_avg(&self->dynamic_window.stat)));
 
@@ -383,12 +383,7 @@ log_source_mangle_hostname(LogSource *self, LogMessage *msg)
       if (G_UNLIKELY(self->options->chain_hostnames))
         {
           msg->flags |= LF_CHAINED_HOSTNAME;
-          if (msg->flags & LF_SIMPLE_HOSTNAME)
-            {
-              /* local without group name */
-              host_len = g_snprintf(host, sizeof(host), "%s", resolved_name);
-            }
-          else if (msg->flags & LF_LOCAL)
+          if (msg->flags & LF_LOCAL)
             {
               /* local */
               host_len = g_snprintf(host, sizeof(host), "%s@%s", self->options->group_name, resolved_name);
@@ -423,15 +418,15 @@ _register_window_stats(LogSource *self)
   const gchar *instance_name = self->name ? : self->stats_instance;
 
   StatsClusterKey sc_key;
-  stats_cluster_single_key_set_with_name(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
-                                         instance_name, "free_window");
+  stats_cluster_single_key_legacy_set_with_name(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
+                                                instance_name, "free_window");
   self->stat_window_size_cluster = stats_register_dynamic_counter(4, &sc_key, SC_TYPE_SINGLE_VALUE,
                                    &self->stat_window_size);
   stats_counter_set(self->stat_window_size, window_size_counter_get(&self->window_size, NULL));
 
 
-  stats_cluster_single_key_set_with_name(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
-                                         instance_name, "full_window");
+  stats_cluster_single_key_legacy_set_with_name(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
+                                                instance_name, "full_window");
   self->stat_full_window_cluster = stats_register_dynamic_counter(4, &sc_key, SC_TYPE_SINGLE_VALUE,
                                    &self->stat_full_window);
   stats_counter_set(self->stat_full_window, self->full_window_size);
@@ -473,7 +468,14 @@ log_source_init(LogPipe *s)
 
   stats_lock();
   StatsClusterKey sc_key;
-  stats_cluster_logpipe_key_set(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id, self->stats_instance);
+  StatsClusterLabel labels[] =
+  {
+    stats_cluster_label("id", self->stats_id),
+    stats_cluster_label("driver_instance", self->stats_instance),
+  };
+  stats_cluster_logpipe_key_set(&sc_key, "input_events_total", labels, G_N_ELEMENTS(labels));
+  stats_cluster_logpipe_key_add_legacy_alias(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
+                                             self->stats_instance);
   stats_register_counter(self->options->stats_level, &sc_key,
                          SC_TYPE_PROCESSED, &self->recvd_messages);
   stats_register_counter(self->options->stats_level, &sc_key, SC_TYPE_STAMP, &self->last_message_seen);
@@ -493,7 +495,14 @@ log_source_deinit(LogPipe *s)
 
   stats_lock();
   StatsClusterKey sc_key;
-  stats_cluster_logpipe_key_set(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id, self->stats_instance);
+  StatsClusterLabel labels[] =
+  {
+    stats_cluster_label("id", self->stats_id),
+    stats_cluster_label("driver_instance", self->stats_instance),
+  };
+  stats_cluster_logpipe_key_set(&sc_key, "input_events_total", labels, G_N_ELEMENTS(labels));
+  stats_cluster_logpipe_key_add_legacy_alias(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
+                                             self->stats_instance);
   stats_unregister_counter(&sc_key, SC_TYPE_PROCESSED, &self->recvd_messages);
   stats_unregister_counter(&sc_key, SC_TYPE_STAMP, &self->last_message_seen);
 
@@ -611,7 +620,7 @@ log_source_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options
   msg_diagnostics(">>>>>> Source side message processing begin",
                   evt_tag_str("instance", self->stats_instance ? self->stats_instance : "internal"),
                   log_pipe_location_tag(s),
-                  evt_tag_printf("msg", "%p", msg));
+                  evt_tag_msg_reference(msg));
 
   /* $HOST setup */
   log_source_mangle_hostname(self, msg);
@@ -660,7 +669,7 @@ log_source_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options
   msg_diagnostics("<<<<<< Source side message processing finish",
                   evt_tag_str("instance", self->stats_instance ? self->stats_instance : "internal"),
                   log_pipe_location_tag(s),
-                  evt_tag_printf("msg", "%p", msg));
+                  evt_tag_msg_reference(msg));
 
   msg_set_context(NULL);
 }
@@ -670,6 +679,8 @@ _initialize_window(LogSource *self, gint init_window_size)
 {
   self->window_initialized = TRUE;
   window_size_counter_set(&self->window_size, init_window_size);
+
+  self->initial_window_size = init_window_size;
   self->full_window_size = init_window_size;
 }
 
