@@ -34,6 +34,7 @@
 #define MAX_RETRIES_ON_ERROR_DEFAULT 3
 #define MAX_RETRIES_BEFORE_SUSPEND_DEFAULT 3
 
+static void _init_stats_legacy_key(LogThreadedDestDriver *self, StatsClusterKey *sc_key);
 static void _init_stats_key(LogThreadedDestDriver *self, StatsClusterKey *sc_key);
 
 const gchar *
@@ -965,12 +966,26 @@ log_threaded_dest_driver_queue(LogPipe *s, LogMessage *msg,
   log_dest_driver_queue_method(s, msg, path_options);
 }
 
+static void
+_init_stats_legacy_key(LogThreadedDestDriver *self, StatsClusterKey *sc_key)
+{
+  stats_cluster_logpipe_key_legacy_set(sc_key, self->stats_source | SCS_DESTINATION,
+                                       self->super.super.id,
+                                       self->format_stats_instance(self));
+}
+
 static void
 _init_stats_key(LogThreadedDestDriver *self, StatsClusterKey *sc_key)
 {
-  stats_cluster_logpipe_key_set(sc_key, self->stats_source | SCS_DESTINATION,
-                                self->super.super.id,
-                                self->format_stats_instance(self));
+  enum { labels_len = 2 };
+  static StatsClusterLabel labels[labels_len];
+
+  labels[0] = stats_cluster_label("id", self->super.super.id ? : "");
+  labels[1] = stats_cluster_label("driver_instance", self->format_stats_instance(self));
+  stats_cluster_logpipe_key_set(sc_key, "output_events_total", labels, labels_len);
+  stats_cluster_logpipe_key_add_legacy_alias(sc_key, self->stats_source | SCS_DESTINATION,
+                                             self->super.super.id,
+                                             self->format_stats_instance(self));
 }
 
 void
@@ -991,28 +1006,28 @@ void
 log_threaded_dest_driver_register_aggregated_stats(LogThreadedDestDriver *self)
 {
   StatsClusterKey sc_key_eps_input;
-  _init_stats_key(self, &sc_key_eps_input);
+  _init_stats_legacy_key(self, &sc_key_eps_input);
   stats_aggregator_lock();
   StatsClusterKey sc_key;
 
-  stats_cluster_single_key_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
-                                         self->format_stats_instance(self), "msg_size_max");
+  stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
+                                                self->format_stats_instance(self), "msg_size_max");
   stats_register_aggregator_maximum(0, &sc_key, &self->max_message_size);
 
-  stats_cluster_single_key_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
-                                         self->format_stats_instance(self), "msg_size_avg");
+  stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
+                                                self->format_stats_instance(self), "msg_size_avg");
   stats_register_aggregator_average(0, &sc_key, &self->average_messages_size);
 
-  stats_cluster_single_key_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
-                                         self->format_stats_instance(self), "batch_size_max");
+  stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
+                                                self->format_stats_instance(self), "batch_size_max");
   stats_register_aggregator_maximum(0, &sc_key, &self->max_batch_size);
 
-  stats_cluster_single_key_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
-                                         self->format_stats_instance(self), "batch_size_avg");
+  stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
+                                                self->format_stats_instance(self), "batch_size_avg");
   stats_register_aggregator_average(0, &sc_key, &self->average_batch_size);
 
-  stats_cluster_single_key_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
-                                         self->format_stats_instance(self), "eps");
+  stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
+                                                self->format_stats_instance(self), "eps");
   stats_register_aggregator_cps(0, &sc_key, &sc_key_eps_input, SC_TYPE_WRITTEN, &self->CPS);
 
   stats_aggregator_unlock();
@@ -1041,9 +1056,12 @@ _register_stats(LogThreadedDestDriver *self)
 
     _init_stats_key(self, &sc_key);
     stats_register_counter(0, &sc_key, SC_TYPE_DROPPED, &self->dropped_messages);
-    stats_register_counter(0, &sc_key, SC_TYPE_PROCESSED, &self->processed_messages);
     stats_register_counter(0, &sc_key, SC_TYPE_WRITTEN, &self->written_messages);
 
+    stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
+                                                  self->format_stats_instance(self), "processed");
+    stats_register_counter(0, &sc_key, SC_TYPE_SINGLE_VALUE, &self->processed_messages);
+
   }
   stats_unlock();
 }
@@ -1057,9 +1075,11 @@ _unregister_stats(LogThreadedDestDriver *self)
 
     _init_stats_key(self, &sc_key);
     stats_unregister_counter(&sc_key, SC_TYPE_DROPPED, &self->dropped_messages);
-    stats_unregister_counter(&sc_key, SC_TYPE_PROCESSED, &self->processed_messages);
     stats_unregister_counter(&sc_key, SC_TYPE_WRITTEN, &self->written_messages);
 
+    stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
+                                                  self->format_stats_instance(self), "processed");
+    stats_unregister_counter(&sc_key, SC_TYPE_SINGLE_VALUE, &self->processed_messages);
   }
   stats_unlock();
 }
@@ -1094,6 +1114,15 @@ _create_workers(LogThreadedDestDriver *self)
   return TRUE;
 }
 
+gboolean
+log_threaded_dest_driver_pre_config_init(LogPipe *s)
+{
+  LogThreadedDestDriver *self = (LogThreadedDestDriver *)s;
+
+  main_loop_worker_allocate_thread_space(self->num_workers);
+  return TRUE;
+}
+
 gboolean
 log_threaded_dest_driver_init_method(LogPipe *s)
 {
@@ -1123,9 +1152,9 @@ log_threaded_dest_driver_init_method(LogPipe *s)
 }
 
 /* This method is only used when a LogThreadedDestDriver is directly used
- * without overriding its on_config_inited method.  If there's an overridden
+ * without overriding its post_config_init method.  If there's an overridden
  * method, the caller is responsible for explicitly calling _start_workers() at
- * the end of on_config_inited(). */
+ * the end of post_config_init(). */
 gboolean
 log_threaded_dest_driver_start_workers(LogPipe *s)
 {
@@ -1183,7 +1212,8 @@ log_threaded_dest_driver_init_instance(LogThreadedDestDriver *self, GlobalConfig
   self->super.super.super.deinit = log_threaded_dest_driver_deinit_method;
   self->super.super.super.queue = log_threaded_dest_driver_queue;
   self->super.super.super.free_fn = log_threaded_dest_driver_free;
-  self->super.super.super.on_config_inited = log_threaded_dest_driver_start_workers;
+  self->super.super.super.pre_config_init = log_threaded_dest_driver_pre_config_init;
+  self->super.super.super.post_config_init = log_threaded_dest_driver_start_workers;
   self->time_reopen = -1;
   self->batch_lines = -1;
   self->batch_timeout = -1;
