@@ -127,7 +127,7 @@ static void
 start_test_threaded_source(TestThreadedSourceDriver *s)
 {
   cr_assert(log_pipe_init(&s->super.super.super.super));
-  cr_assert(log_pipe_on_config_inited(&s->super.super.super.super));
+  cr_assert(log_pipe_post_config_init(&s->super.super.super.super));
 }
 
 static void
