@@ -23,7 +23,7 @@
  */
 #include "mainloop.h"
 #include "control/control-commands.h"
-#include "control/control-server.h"
+#include "control/control-connection.h"
 #include "messages.h"
 #include "cfg.h"
 #include "cfg-path.h"
@@ -34,40 +34,69 @@
 
 #include <string.h>
 
+static gboolean
+_control_process_log_level(const gchar *level, GString *result)
+{
+  if (!level)
+    {
+      /* query current log level */
+      return TRUE;
+    }
+  gint ll = msg_map_string_to_log_level(level);
+  if (ll < 0)
+    return FALSE;
+
+  msg_set_log_level(ll);
+  return TRUE;
+}
+
+static gboolean
+_control_process_compat_log_command(const gchar *level, const gchar *onoff, GString *result)
+{
+  if (!level)
+    return FALSE;
+
+  gint ll = msg_map_string_to_log_level(level);
+  if (ll < 0)
+    return FALSE;
+
+  if (onoff)
+    {
+      gboolean on = g_str_equal(onoff, "ON");
+      if (!on)
+        ll = ll - 1;
+
+      msg_set_log_level(ll);
+    }
+  return TRUE;
+}
+
 static void
-control_connection_message_log(ControlConnection *cc, GString *command, gpointer user_data)
+control_connection_message_log(ControlConnection *cc, GString *command, gpointer user_data, gboolean *cancelled)
 {
   gchar **cmds = g_strsplit(command->str, " ", 3);
-  gboolean on;
-  int *type = NULL;
   GString *result = g_string_sized_new(128);
+  gboolean success;
 
   if (!cmds[1])
     {
-      g_string_assign(result, "Invalid arguments received, expected at least one argument");
+      g_string_assign(result, "FAIL Invalid arguments received");
       goto exit;
     }
 
-  if (g_str_equal(cmds[1], "DEBUG"))
-    type = &debug_flag;
-  else if (g_str_equal(cmds[1], "VERBOSE"))
-    type = &verbose_flag;
-  else if (g_str_equal(cmds[1], "TRACE"))
-    type = &trace_flag;
+  gint orig_log_level = msg_get_log_level();
 
-  if (type)
-    {
-      if (cmds[2])
-        {
-          on = g_str_equal(cmds[2], "ON");
-          if (*type != on)
-            {
-              msg_info("Verbosity changed", evt_tag_str("type", cmds[1]), evt_tag_int("on", on));
-              *type = on;
-            }
-        }
-      g_string_printf(result, "OK %s=%d", cmds[1], *type);
-    }
+  if (g_str_equal(cmds[1], "LEVEL"))
+    success = _control_process_log_level(cmds[2], result);
+  else
+    success = _control_process_compat_log_command(cmds[1], cmds[2], result);
+
+  if (orig_log_level != msg_get_log_level())
+    msg_info("Verbosity changed",
+             evt_tag_int("log_level", msg_get_log_level()));
+
+  if (success)
+    g_string_printf(result, "OK syslog-ng log level set to %d", msg_get_log_level());
   else
     g_string_assign(result, "FAIL Invalid arguments received");
 exit:
@@ -76,7 +105,7 @@ exit:
 }
 
 static void
-control_connection_stop_process(ControlConnection *cc, GString *command, gpointer user_data)
+control_connection_stop_process(ControlConnection *cc, GString *command, gpointer user_data, gboolean *cancelled)
 {
   GString *result = g_string_new("OK Shutdown initiated");
   MainLoop *main_loop = (MainLoop *) user_data;
@@ -87,13 +116,19 @@ control_connection_stop_process(ControlConnection *cc, GString *command, gpointe
 }
 
 static void
-control_connection_config(ControlConnection *cc, GString *command, gpointer user_data)
+control_connection_config(ControlConnection *cc, GString *command, gpointer user_data, gboolean *cancelled)
 {
   MainLoop *main_loop = (MainLoop *) user_data;
   GlobalConfig *config = main_loop_get_current_config(main_loop);
   GString *result = g_string_sized_new(128);
   gchar **arguments = g_strsplit(command->str, " ", 0);
 
+  if (!arguments[1])
+    {
+      g_string_assign(result, "FAIL Invalid arguments");
+      goto exit;
+    }
+
   if (g_str_equal(arguments[1], "GET"))
     {
       if (g_str_equal(arguments[2], "ORIGINAL"))
@@ -122,7 +157,7 @@ exit:
 }
 
 static void
-show_ose_license_info(ControlConnection *cc, GString *command, gpointer user_data)
+show_ose_license_info(ControlConnection *cc, GString *command, gpointer user_data, gboolean *cancelled)
 {
   control_connection_send_reply(cc,
                                 g_string_new("OK You are using the Open Source Edition of syslog-ng."));
@@ -145,7 +180,7 @@ _respond_config_reload_status(gint type, gpointer user_data)
 }
 
 static void
-control_connection_reload(ControlConnection *cc, GString *command, gpointer user_data)
+control_connection_reload(ControlConnection *cc, GString *command, gpointer user_data, gboolean *cancelled)
 {
   MainLoop *main_loop = (MainLoop *) user_data;
   static gpointer args[2];
@@ -168,7 +203,7 @@ control_connection_reload(ControlConnection *cc, GString *command, gpointer user
 }
 
 static void
-control_connection_reopen(ControlConnection *cc, GString *command, gpointer user_data)
+control_connection_reopen(ControlConnection *cc, GString *command, gpointer user_data, gboolean *cancelled)
 {
   GString *result = g_string_new("OK Re-open of log destination files initiated");
   app_reopen_files();
@@ -232,7 +267,7 @@ process_credentials_add(GString *result, guint argc, gchar **arguments)
 }
 
 static void
-process_credentials(ControlConnection *cc, GString *command, gpointer user_data)
+process_credentials(ControlConnection *cc, GString *command, gpointer user_data, gboolean *cancelled)
 {
   gchar **arguments = g_strsplit(command->str, " ", 4);
   guint argc = g_strv_length(arguments);
@@ -261,7 +296,7 @@ process_credentials(ControlConnection *cc, GString *command, gpointer user_data)
 }
 
 static void
-control_connection_list_files(ControlConnection *cc, GString *command, gpointer user_data)
+control_connection_list_files(ControlConnection *cc, GString *command, gpointer user_data, gboolean *cancelled)
 {
   MainLoop *main_loop = (MainLoop *) user_data;
   GlobalConfig *config = main_loop_get_current_config(main_loop);
@@ -394,7 +429,7 @@ generate_json(GHashTable *nodes, GHashTable *arcs)
 }
 
 static void
-export_config_graph(ControlConnection *cc, GString *command, gpointer user_data)
+export_config_graph(ControlConnection *cc, GString *command, gpointer user_data, gboolean *cancelled)
 {
   GHashTable *nodes;
   GHashTable *arcs;
@@ -432,6 +467,6 @@ main_loop_register_control_commands(MainLoop *main_loop)
   for (i = 0; default_commands[i].command_name != NULL; i++)
     {
       cmd = &default_commands[i];
-      control_register_command(cmd->command_name, cmd->func, main_loop);
+      control_register_command(cmd->command_name, cmd->func, main_loop, cmd->threaded);
     }
 }
