@@ -132,6 +132,12 @@ extern int pragma_debug;
     KW_BATCH_LINES = 10087,        /* KW_BATCH_LINES  */
     KW_BATCH_TIMEOUT = 10088,      /* KW_BATCH_TIMEOUT  */
     KW_TRIM_LARGE_MESSAGES = 10089, /* KW_TRIM_LARGE_MESSAGES  */
+    KW_STATS = 10400,              /* KW_STATS  */
+    KW_FREQ = 10401,               /* KW_FREQ  */
+    KW_LEVEL = 10402,              /* KW_LEVEL  */
+    KW_LIFETIME = 10403,           /* KW_LIFETIME  */
+    KW_MAX_DYNAMIC = 10404,        /* KW_MAX_DYNAMIC  */
+    KW_SYSLOG_STATS = 10405,       /* KW_SYSLOG_STATS  */
     KW_CHAIN_HOSTNAMES = 10090,    /* KW_CHAIN_HOSTNAMES  */
     KW_NORMALIZE_HOSTNAMES = 10091, /* KW_NORMALIZE_HOSTNAMES  */
     KW_KEEP_HOSTNAME = 10092,      /* KW_KEEP_HOSTNAME  */
@@ -161,10 +167,10 @@ extern int pragma_debug;
     KW_HOST_OVERRIDE = 10166,      /* KW_HOST_OVERRIDE  */
     KW_THROTTLE = 10170,           /* KW_THROTTLE  */
     KW_THREADED = 10171,           /* KW_THREADED  */
-    KW_PASS_UNIX_CREDENTIALS = 10231, /* KW_PASS_UNIX_CREDENTIALS  */
-    KW_PERSIST_NAME = 10302,       /* KW_PERSIST_NAME  */
-    KW_READ_OLD_RECORDS = 10304,   /* KW_READ_OLD_RECORDS  */
-    KW_USE_SYSLOGNG_PID = 10305,   /* KW_USE_SYSLOGNG_PID  */
+    KW_PASS_UNIX_CREDENTIALS = 10180, /* KW_PASS_UNIX_CREDENTIALS  */
+    KW_PERSIST_NAME = 10181,       /* KW_PERSIST_NAME  */
+    KW_READ_OLD_RECORDS = 10182,   /* KW_READ_OLD_RECORDS  */
+    KW_USE_SYSLOGNG_PID = 10183,   /* KW_USE_SYSLOGNG_PID  */
     KW_FLAGS = 10190,              /* KW_FLAGS  */
     KW_PAD_SIZE = 10200,           /* KW_PAD_SIZE  */
     KW_TIME_ZONE = 10201,          /* KW_TIME_ZONE  */
@@ -190,6 +196,7 @@ extern int pragma_debug;
     KW_TEMPLATE_FUNCTION = 10272,  /* KW_TEMPLATE_FUNCTION  */
     KW_DEFAULT_FACILITY = 10300,   /* KW_DEFAULT_FACILITY  */
     KW_DEFAULT_SEVERITY = 10301,   /* KW_DEFAULT_SEVERITY  */
+    KW_SDATA_PREFIX = 10302,       /* KW_SDATA_PREFIX  */
     KW_PORT = 10323,               /* KW_PORT  */
     KW_USE_TIME_RECVD = 10340,     /* KW_USE_TIME_RECVD  */
     KW_FACILITY = 10350,           /* KW_FACILITY  */
@@ -205,6 +212,7 @@ extern int pragma_debug;
     KW_VALUE = 10372,              /* KW_VALUE  */
     KW_YES = 10380,                /* KW_YES  */
     KW_NO = 10381,                 /* KW_NO  */
+    KW_AUTO = 10382,               /* KW_AUTO  */
     KW_IFDEF = 10410,              /* KW_IFDEF  */
     KW_ENDIF = 10411,              /* KW_ENDIF  */
     LL_DOTDOT = 10420,             /* LL_DOTDOT  */
@@ -238,7 +246,8 @@ extern int pragma_debug;
     KW_VERSION_CURRENT = 10525,    /* KW_VERSION_CURRENT  */
     KW_DEFINE = 10526,             /* KW_DEFINE  */
     KW_MODULE = 10527,             /* KW_MODULE  */
-    KW_REQUIRES = 10528            /* KW_REQUIRES  */
+    KW_REQUIRES = 10528,           /* KW_REQUIRES  */
+    KW_LINE = 10529                /* KW_LINE  */
   };
   typedef enum pragma_tokentype pragma_token_kind_t;
 #endif
@@ -302,6 +311,12 @@ extern int pragma_debug;
 #define KW_BATCH_LINES 10087
 #define KW_BATCH_TIMEOUT 10088
 #define KW_TRIM_LARGE_MESSAGES 10089
+#define KW_STATS 10400
+#define KW_FREQ 10401
+#define KW_LEVEL 10402
+#define KW_LIFETIME 10403
+#define KW_MAX_DYNAMIC 10404
+#define KW_SYSLOG_STATS 10405
 #define KW_CHAIN_HOSTNAMES 10090
 #define KW_NORMALIZE_HOSTNAMES 10091
 #define KW_KEEP_HOSTNAME 10092
@@ -331,10 +346,10 @@ extern int pragma_debug;
 #define KW_HOST_OVERRIDE 10166
 #define KW_THROTTLE 10170
 #define KW_THREADED 10171
-#define KW_PASS_UNIX_CREDENTIALS 10231
-#define KW_PERSIST_NAME 10302
-#define KW_READ_OLD_RECORDS 10304
-#define KW_USE_SYSLOGNG_PID 10305
+#define KW_PASS_UNIX_CREDENTIALS 10180
+#define KW_PERSIST_NAME 10181
+#define KW_READ_OLD_RECORDS 10182
+#define KW_USE_SYSLOGNG_PID 10183
 #define KW_FLAGS 10190
 #define KW_PAD_SIZE 10200
 #define KW_TIME_ZONE 10201
@@ -360,6 +375,7 @@ extern int pragma_debug;
 #define KW_TEMPLATE_FUNCTION 10272
 #define KW_DEFAULT_FACILITY 10300
 #define KW_DEFAULT_SEVERITY 10301
+#define KW_SDATA_PREFIX 10302
 #define KW_PORT 10323
 #define KW_USE_TIME_RECVD 10340
 #define KW_FACILITY 10350
@@ -375,6 +391,7 @@ extern int pragma_debug;
 #define KW_VALUE 10372
 #define KW_YES 10380
 #define KW_NO 10381
+#define KW_AUTO 10382
 #define KW_IFDEF 10410
 #define KW_ENDIF 10411
 #define LL_DOTDOT 10420
@@ -409,6 +426,7 @@ extern int pragma_debug;
 #define KW_DEFINE 10526
 #define KW_MODULE 10527
 #define KW_REQUIRES 10528
+#define KW_LINE 10529
 
 /* Value type.  */
 #if ! defined PRAGMA_STYPE && ! defined PRAGMA_STYPE_IS_DECLARED
