@@ -20,6 +20,10 @@
  *
  */
 
+#include <criterion/criterion.h>
+#include "libtest/queue_utils_lib.h"
+#include "test_diskq_tools.h"
+
 #include "logqueue.h"
 #include "logqueue-disk.h"
 #include "logqueue-disk-reliable.h"
@@ -29,12 +33,9 @@
 #include "apphook.h"
 #include "diskq-config.h"
 
-#include "queue_utils_lib.h"
-#include "test_diskq_tools.h"
-#include <criterion/criterion.h>
-
 #include <stdlib.h>
 #include <sys/stat.h>
+#include <math.h>
 
 #define TEST_DISKQ_SIZE 1100000
 
@@ -74,12 +75,40 @@ _save_diskqueue(LogQueue *q)
   log_queue_unref(q);
 }
 
+static gsize
+_calculate_serialized_empty_message_size(LogQueueDisk *queue_disk)
+{
+  LogMessage *msg = log_msg_new_empty();
+  GString *serialized_msg = g_string_new(NULL);
+
+  cr_assert(log_queue_disk_serialize_msg(queue_disk, msg, serialized_msg));
+  gsize size = serialized_msg->len;
+
+  g_string_free(serialized_msg, TRUE);
+  log_msg_unref(msg);
+  return size;
+}
+
+static gint64
+_calculate_full_disk_message_num(LogQueueDisk *queue_disk)
+{
+  /*
+   * Assumes that only empty messages are stored.
+   */
+  QDisk *qdisk = queue_disk->qdisk;
+
+  gsize msg_size = _calculate_serialized_empty_message_size(queue_disk);
+  gint num_messages = llrint(ceil((qdisk->options->disk_buf_size - QDISK_RESERVED_SPACE) / (double) msg_size));
+
+  return num_messages;
+}
+
 static gint64
 _get_file_size(LogQueue *q)
 {
   struct stat file_stats;
   const gchar *filename = log_queue_disk_get_filename(q);
-  assert_gint(stat(filename, &file_stats), 0, "Stat call failed, errno:%d", errno);
+  cr_assert(stat(filename, &file_stats) == 0, "Stat call failed, errno:%d", errno);
   return (gint64)file_stats.st_size;
 }
 
@@ -222,18 +251,20 @@ _create_reliable_diskqueue(gchar *filename, DiskQueueOptions *options, gboolean
 
 Test(diskq_truncate, test_diskq_truncate_on_push)
 {
-  const gint full_disk_message_number = 8194; // measured for disk_buf_size TEST_DISKQ_SIZE
-  const gint read_is_on_end_message_number  = full_disk_message_number - 200;
-  const gint write_wraps_message_number = read_is_on_end_message_number - 200;
-  const gint read_wraps_message_number = 300;
-  const gint trigger_truncate_message_number = 1;
   LogQueue *q;
   GString *filename = g_string_new("test_dq_truncate_on_write.rqf");
 
   DiskQueueOptions options;
-  q = _create_reliable_diskqueue(filename->str, &options, FALSE, -1);
+  q = _create_reliable_diskqueue(filename->str, &options, FALSE, 0);
   cr_assert_eq(log_queue_get_length(q), 0, "No messages should be in a newly created disk-queue file!");
 
+  LogQueueDisk *queue_disk = (LogQueueDisk *)q;
+  const gint full_disk_message_number = _calculate_full_disk_message_num(queue_disk);
+  const gint read_is_on_end_message_number  = full_disk_message_number - 200;
+  const gint write_wraps_message_number = read_is_on_end_message_number - 200;
+  const gint read_wraps_message_number = 300;
+  const gint trigger_truncate_message_number = 1;
+
   // 1. fill it until its FULL
   feed_some_messages(q, full_disk_message_number);
   cr_assert_eq(log_queue_get_length(q), full_disk_message_number,
@@ -282,20 +313,23 @@ _assert_cursors_are_at_start(LogQueue *q)
   cr_assert_eq(qdisk_get_backlog_head(qdisk), QDISK_RESERVED_SPACE, "Backlog head was not reset!");
 }
 
-Test(diskq_truncate, test_diskq_truncate_size_ratio_default)
+Test(diskq_truncate, test_diskq_truncate_size_ratio_default_3_x)
 {
-  const gint truncate_threshold = (gint)(TEST_DISKQ_SIZE * disk_queue_config_get_truncate_size_ratio(configuration));
-  const gint empty_log_msg_size = 134;
-  const gint below_threshold_message_number = truncate_threshold / empty_log_msg_size;
-  const gint above_threshold_message_number = below_threshold_message_number + 1;
+  cfg_set_version_without_validation(configuration, VERSION_VALUE_3_38);
 
   LogQueue *q;
-  GString *filename = g_string_new("test_dq_truncate_size_ratio_default.rqf");
+  GString *filename = g_string_new("test_dq_truncate_size_ratio_default_3_x.rqf");
 
   DiskQueueOptions options;
   q = _create_reliable_diskqueue(filename->str, &options, TRUE, -1);
   cr_assert_eq(log_queue_get_length(q), 0, "No messages should be in a newly created disk-queue file!");
 
+  LogQueueDisk *queue_disk = (LogQueueDisk *)q;
+  const gint truncate_threshold = (gint)(TEST_DISKQ_SIZE * disk_queue_config_get_truncate_size_ratio(configuration));
+  const gint empty_log_msg_size = _calculate_serialized_empty_message_size(queue_disk);
+  const gint below_threshold_message_number = truncate_threshold / empty_log_msg_size;
+  const gint above_threshold_message_number = below_threshold_message_number + 1;
+
   // 1. fill it just below the truncate threshold
   feed_some_messages(q, below_threshold_message_number);
   cr_assert_eq(log_queue_get_length(q), below_threshold_message_number,
@@ -357,9 +391,46 @@ Test(diskq_truncate, test_diskq_truncate_size_ratio_0)
   _save_diskqueue(q);
 }
 
+Test(diskq_truncate, test_diskq_truncate_short_read)
+{
+  gint FIRST_ACK_NUMBER = 10;
+  gint SEND_AND_ACK_MSG_NUMBER = 5;
+  LogQueue *q;
+  GString *filename = g_string_new("test_diskq_truncate_short_read.rqf");
+
+  DiskQueueOptions options;
+  q = _create_reliable_diskqueue(filename->str, &options, TRUE, 1);
+  cr_assert_eq(log_queue_get_length(q), 0, "No messages should be in a newly created disk-queue file!");
+
+  LogQueueDisk *disk_queue = (LogQueueDisk *) q;
+  const gint full_disk_message_number = _calculate_full_disk_message_num(disk_queue);
+  // 1. feed to full
+  feed_some_messages(q, full_disk_message_number);
+  cr_assert_eq(log_queue_get_length(q), full_disk_message_number,
+               "Not all messages have been queued!");
+
+  // 2. process some messages
+  send_some_messages(q, full_disk_message_number);
+  log_queue_ack_backlog(q, FIRST_ACK_NUMBER);
+
+  // 3. send and ack more messages, but qdisk is not full
+  feed_some_messages(q, SEND_AND_ACK_MSG_NUMBER);
+  send_some_messages(q, SEND_AND_ACK_MSG_NUMBER);
+  log_queue_ack_backlog(q, SEND_AND_ACK_MSG_NUMBER);
+
+  // 4. ack every messages
+  log_queue_ack_backlog(q, full_disk_message_number - FIRST_ACK_NUMBER);
+
+  _assert_diskq_actual_file_size_with_stored(q);
+
+  unlink(filename->str);
+  g_string_free(filename, TRUE);
+
+  _save_diskqueue(q);
+}
+
 Test(diskq_truncate, test_diskq_truncate_size_ratio_1)
 {
-  const gint full_disk_message_number = 8194; // measured for disk_buf_size TEST_DISKQ_SIZE
   LogQueue *q;
   GString *filename = g_string_new("test_dq_truncate_size_ratio_1.rqf");
 
@@ -367,6 +438,9 @@ Test(diskq_truncate, test_diskq_truncate_size_ratio_1)
   q = _create_reliable_diskqueue(filename->str, &options, TRUE, 1);
   cr_assert_eq(log_queue_get_length(q), 0, "No messages should be in a newly created disk-queue file!");
 
+  LogQueueDisk *queue_disk = (LogQueueDisk *)q;
+  const gint full_disk_message_number = _calculate_full_disk_message_num(queue_disk);
+
   // 1. feed to full
   feed_some_messages(q, full_disk_message_number);
   cr_assert_eq(log_queue_get_length(q), full_disk_message_number,
@@ -405,8 +479,6 @@ _feed_one_large_message(LogQueue *q)
 
 Test(diskq_truncate, test_diskq_no_truncate_wrap)
 {
-  const gint just_under_max_size_message_number = 8239; // measured for disk_buf_size TEST_DISKQ_SIZE
-  const gint some_messages_to_let_write_head_wrap = 500;
   gint unprocessed_messages_in_buffer = 0;
   LogQueue *q;
   GString *filename = g_string_new("test_dq_no_truncate_wrap.rqf");
@@ -415,7 +487,9 @@ Test(diskq_truncate, test_diskq_no_truncate_wrap)
   q = _create_reliable_diskqueue(filename->str, &options, TRUE, 1);
   cr_assert_eq(log_queue_get_length(q), 0, "No messages should be in a newly created disk-queue file!");
 
-  QDisk *qdisk = ((LogQueueDisk *)q)->qdisk;
+  LogQueueDisk *queue_disk = (LogQueueDisk *)q;
+  const gint just_under_max_size_message_number = _calculate_full_disk_message_num(queue_disk) - 1;
+  const gint some_messages_to_let_write_head_wrap = 500;
 
   // 1. feed to full
   feed_some_messages(q, just_under_max_size_message_number);
@@ -430,6 +504,7 @@ Test(diskq_truncate, test_diskq_no_truncate_wrap)
 
   // 3. feed 1 large msg to make file_size big
   _feed_one_large_message(q);
+  QDisk *qdisk = ((LogQueueDisk *)q)->qdisk;
   cr_assert(qdisk_get_writer_head(qdisk) < qdisk_get_reader_head(qdisk), "write_head should have wrapped");
   cr_assert(qdisk->file_size > TEST_DISKQ_SIZE, "file_size should be bigger than max size");
   unprocessed_messages_in_buffer += 1;
@@ -476,8 +551,6 @@ Test(diskq_truncate, test_diskq_no_truncate_wrap)
 Test(diskq_truncate, test_non_reliable_diskq_restart_with_truncation_disabled)
 {
   const gint qout_size = 128;
-  const gint just_under_max_size_message_number = 8239 + qout_size; // measured for disk_buf_size TEST_DISKQ_SIZE
-  const gint some_messages_to_let_write_head_wrap = 500;
 
   GString *filename = g_string_new("test_dq_non_reliable_restart.rqf");
 
@@ -485,7 +558,9 @@ Test(diskq_truncate, test_non_reliable_diskq_restart_with_truncation_disabled)
   LogQueue *q = _create_non_reliable_diskqueue(filename->str, &options, qout_size, 1);
   cr_assert_eq(log_queue_get_length(q), 0, "No messages should be in a newly created disk-queue file!");
 
-  QDisk *qdisk = ((LogQueueDisk *)q)->qdisk;
+  LogQueueDisk *queue_disk = (LogQueueDisk *)q;
+  const gint just_under_max_size_message_number = (_calculate_full_disk_message_num(queue_disk) + qout_size) - 1;
+  const gint some_messages_to_let_write_head_wrap = 500;
 
   // 1. feed to full
   feed_some_messages(q, just_under_max_size_message_number);
@@ -499,6 +574,7 @@ Test(diskq_truncate, test_non_reliable_diskq_restart_with_truncation_disabled)
   // 3. feed additional messages, so write_head wraps
   feed_some_messages(q, 300);
   unprocessed_messages_in_buffer += 300;
+  QDisk *qdisk = ((LogQueueDisk *)q)->qdisk;
   cr_assert(qdisk_get_writer_head(qdisk) < qdisk_get_reader_head(qdisk), "write_head should have wrapped");
 
   // Restart
