@@ -173,6 +173,20 @@ http_dd_set_cipher_suite(LogDriver *d, const gchar *ciphers)
   self->ciphers = g_strdup(ciphers);
 }
 
+gboolean
+http_dd_set_tls13_cipher_suite(LogDriver *d, const gchar *tls13_ciphers)
+{
+#if SYSLOG_NG_HAVE_DECL_CURLOPT_TLS13_CIPHERS
+  HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
+
+  g_free(self->tls13_ciphers);
+  self->tls13_ciphers = g_strdup(tls13_ciphers);
+  return TRUE;
+#else
+  return FALSE;
+#endif
+}
+
 void
 http_dd_set_proxy(LogDriver *d, const gchar *proxy)
 {
@@ -255,6 +269,19 @@ http_dd_set_peer_verify(LogDriver *d, gboolean verify)
   self->peer_verify = verify;
 }
 
+gboolean
+http_dd_set_ocsp_stapling_verify(LogDriver *d, gboolean verify)
+{
+#if SYSLOG_NG_HAVE_DECL_CURLOPT_SSL_VERIFYSTATUS
+  HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
+
+  self->ocsp_stapling_verify = verify;
+  return TRUE;
+#else
+  return FALSE;
+#endif
+}
+
 void
 http_dd_set_accept_redirects(LogDriver *d, gboolean accept_redirects)
 {
@@ -392,9 +419,9 @@ http_dd_free(LogPipe *s)
   g_free(self->cert_file);
   g_free(self->key_file);
   g_free(self->ciphers);
+  g_free(self->tls13_ciphers);
   g_free(self->proxy);
   g_list_free_full(self->headers, g_free);
-  g_mutex_free(self->workers_lock);
   http_load_balancer_free(self->load_balancer);
   http_response_handlers_free(self->response_handlers);
 
@@ -427,7 +454,6 @@ http_dd_new(GlobalConfig *cfg)
   self->body_prefix = g_string_new("");
   self->body_suffix = g_string_new("");
   self->delimiter = g_string_new("\n");
-  self->workers_lock = g_mutex_new();
   self->load_balancer = http_load_balancer_new();
   curl_version_info_data *curl_info = curl_version_info(CURLVERSION_NOW);
   if (!self->user_agent)
