@@ -96,7 +96,9 @@ JNIEXPORT jlong JNICALL
 Java_org_syslog_1ng_LogPipe_getConfigHandle(JNIEnv *env, jobject obj, jlong handle)
 {
   JavaDestDriver *self = (JavaDestDriver *)handle;
-  return (jlong)log_pipe_get_config(&self->super.super.super.super);
+  GlobalConfig *cfg = log_pipe_get_config(&self->super.super.super.super);
+
+  return (jlong) cfg;
 }
 
 void java_dd_set_option(LogDriver *s, const gchar *key, const gchar *value)
@@ -137,9 +139,6 @@ java_dd_init(LogPipe *s)
   GlobalConfig *cfg = log_pipe_get_config(s);
   GError *error = NULL;
 
-  if (!log_threaded_dest_driver_init_method(s))
-    return FALSE;
-
   log_template_options_init(&self->template_options, cfg);
 
   if (!log_template_compile(self->template, self->template_string, &error))
@@ -160,6 +159,9 @@ java_dd_init(LogPipe *s)
   if (!java_destination_proxy_init(self->proxy))
     return FALSE;
 
+  if (!log_threaded_dest_driver_init_method(s))
+    return FALSE;
+
   return TRUE;
 }
 
