@@ -19,10 +19,12 @@
  * OpenSSL libraries as published by the OpenSSL project. See the file
  * COPYING for details.
  */
+
+#include <criterion/criterion.h>
+#include "libtest/msg_parse_lib.h"
+
 #include "json-parser.h"
 #include "apphook.h"
-#include "msg_parse_lib.h"
-#include <criterion/criterion.h>
 
 static LogMessage *
 parse_json_into_log_message_no_check(const gchar *json, LogParser *json_parser)
@@ -99,6 +101,19 @@ Test(json_parser, test_json_parser_adds_prefix_to_name_value_pairs_when_instruct
   log_pipe_unref(&json_parser->super);
 }
 
+Test(json_parser, test_json_parser_uses_key_delimiter_when_its_specified)
+{
+  LogMessage *msg;
+  LogParser *json_parser = json_parser_new(NULL);
+
+  json_parser_set_key_delimiter(json_parser, '\t');
+  msg = parse_json_into_log_message("{'foo': 'bar', 'embed': {'foo': 'bar'}}", json_parser);
+  assert_log_message_value(msg, log_msg_get_value_handle("foo"), "bar");
+  assert_log_message_value(msg, log_msg_get_value_handle("embed\tfoo"), "bar");
+  log_msg_unref(msg);
+  log_pipe_unref(&json_parser->super);
+}
+
 Test(json_parser, test_json_parser_skips_marker_when_set_in_the_input)
 {
   LogMessage *msg;
@@ -134,15 +149,53 @@ Test(json_parser, test_json_parser_validate_type_representation)
   json_parser_set_prefix(json_parser, ".prefix.");
   msg = parse_json_into_log_message("{'int': 123, 'booltrue': true, 'boolfalse': false, 'double': 1.23, 'object': {'member1': 'foo', 'member2': 'bar'}, 'array': [1, 2, 3], 'null': null}",
                                     json_parser);
-  assert_log_message_value(msg, log_msg_get_value_handle(".prefix.int"), "123");
-  assert_log_message_value(msg, log_msg_get_value_handle(".prefix.booltrue"), "true");
-  assert_log_message_value(msg, log_msg_get_value_handle(".prefix.boolfalse"), "false");
-  assert_log_message_value(msg, log_msg_get_value_handle(".prefix.double"), "1.230000");
-  assert_log_message_value(msg, log_msg_get_value_handle(".prefix.object.member1"), "foo");
-  assert_log_message_value(msg, log_msg_get_value_handle(".prefix.object.member2"), "bar");
-  assert_log_message_value(msg, log_msg_get_value_handle(".prefix.array[0]"), "1");
-  assert_log_message_value(msg, log_msg_get_value_handle(".prefix.array[1]"), "2");
-  assert_log_message_value(msg, log_msg_get_value_handle(".prefix.array[2]"), "3");
+  assert_log_message_value_and_type_by_name(msg, ".prefix.int", "123", LM_VT_INTEGER);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.booltrue", "true", LM_VT_BOOLEAN);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.boolfalse", "false", LM_VT_BOOLEAN);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.double", "1.230000", LM_VT_DOUBLE);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.object.member1", "foo", LM_VT_STRING);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.object.member2", "bar", LM_VT_STRING);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.array", "1,2,3", LM_VT_LIST);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.null", "", LM_VT_NULL);
+  log_msg_unref(msg);
+  log_pipe_unref(&json_parser->super);
+}
+
+Test(json_parser, test_json_parser_different_type_arrays)
+{
+  LogMessage *msg;
+  LogParser *json_parser = json_parser_new(NULL);
+
+  json_parser_set_prefix(json_parser, ".prefix.");
+  msg = parse_json_into_log_message("{'intarray': [1, 2, 3],"
+                                    " 'strarray': ['foo', 'bar', 'baz'],"
+                                    " 'boolarray': [true,false,true],"
+                                    " 'dblarray': [1.234,1e6,5.6789],"
+                                    " 'nullarray': [null,null,null,null]}",
+                                    json_parser);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.intarray", "1,2,3", LM_VT_LIST);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.strarray", "foo,bar,baz", LM_VT_LIST);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.boolarray", "true,false,true", LM_VT_LIST);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.dblarray", "1.234000,1000000.000000,5.678900", LM_VT_LIST);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.nullarray", "\"\",\"\",\"\",\"\"", LM_VT_LIST);
+  log_msg_unref(msg);
+  log_pipe_unref(&json_parser->super);
+}
+
+Test(json_parser, test_json_parser_compound_types_in_arrays_are_represented_as_json_encoded_strings)
+{
+  LogMessage *msg;
+  LogParser *json_parser = json_parser_new(NULL);
+
+  json_parser_set_prefix(json_parser, ".prefix.");
+  msg = parse_json_into_log_message("{'arrayofarrays': [[1,2],[3,4],[5,6]],"
+                                    " 'arrayofmixedtypes': ['str',42,{},null],"
+                                    " 'arrayofobjects': [{'foo':'bar','bar':'foo'},{'foo':'bar','bar':'foo'}]}",
+                                    json_parser);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.arrayofarrays", "[[1,2],[3,4],[5,6]]", LM_VT_JSON);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.arrayofmixedtypes", "[\"str\",42,{},null]", LM_VT_JSON);
+  assert_log_message_value_and_type_by_name(msg, ".prefix.arrayofobjects",
+                                            "[{\"foo\":\"bar\",\"bar\":\"foo\"},{\"foo\":\"bar\",\"bar\":\"foo\"}]", LM_VT_JSON);
   log_msg_unref(msg);
   log_pipe_unref(&json_parser->super);
 }
@@ -177,11 +230,12 @@ Test(json_parser, test_json_parser_int64_mid)
   log_pipe_unref(&json_parser->super);
 }
 
-Test(json_parser, test_json_parser_fails_for_non_object_top_element)
+Test(json_parser, test_json_parser_fails_for_non_object_and_non_array_top_element)
 {
   LogParser *json_parser = json_parser_new(NULL);
-  assert_json_parser_fails("[1, 2, 3]", json_parser);
-  assert_json_parser_fails("", json_parser);
+  assert_json_parser_fails("true", json_parser);
+  assert_json_parser_fails("null", json_parser);
+  assert_json_parser_fails("10", json_parser);
   log_pipe_unref(&json_parser->super);
 }
 
@@ -197,13 +251,30 @@ Test(json_parser, test_json_parser_extracts_subobjects_if_extract_prefix_is_spec
   log_pipe_unref(&json_parser->super);
 }
 
+Test(json_parser, test_json_parser_extracts_array_elements_into_matches)
+{
+  LogMessage *msg;
+  LogParser *json_parser = json_parser_new(NULL);
+
+  msg = parse_json_into_log_message("[42,true,null,{'foo':'bar'}, {'bar':'foo'}]", json_parser);
+  assert_log_message_value_unset_by_name(msg, "0");
+  assert_log_message_value_and_type_by_name(msg, "1", "42", LM_VT_INTEGER);
+  assert_log_message_value_and_type_by_name(msg, "2", "true", LM_VT_BOOLEAN);
+  assert_log_message_value_and_type_by_name(msg, "3", "", LM_VT_NULL);
+  assert_log_message_value_and_type_by_name(msg, "4", "{\"foo\":\"bar\"}", LM_VT_JSON);
+  assert_log_message_value_and_type_by_name(msg, "5", "{\"bar\":\"foo\"}", LM_VT_JSON);
+  cr_assert(msg->num_matches == 6);
+  log_msg_unref(msg);
+  log_pipe_unref(&json_parser->super);
+}
+
 Test(json_parser, test_json_parser_works_with_templates)
 {
   LogMessage *msg;
   LogTemplate *template;
 
   template = log_template_new(NULL, NULL);
-  log_template_compile(template, "{'foo':'bar'}", NULL);
+  log_template_compile_literal_string(template, "{'foo':'bar'}");
 
   LogParser *json_parser = json_parser_new(NULL);
   log_parser_set_template(json_parser, template);
