@@ -29,6 +29,7 @@
 #include "python-logtemplate-options.h"
 #include "python-integerpointer.h"
 #include "python-helpers.h"
+#include "python-types.h"
 #include "logthrdest/logthrdestdrv.h"
 #include "stats/stats.h"
 #include "string-list.h"
@@ -55,13 +56,24 @@ typedef struct
     PyObject *open;
     PyObject *send;
     PyObject *flush;
-    PyObject *log_template_options;
-    PyObject *seqnum;
     PyObject *generate_persist_name;
     GPtrArray *_refs_to_clean;
   } py;
 } PythonDestDriver;
 
+typedef struct _PyLogDestination
+{
+  PyObject_HEAD
+} PyLogDestination;
+
+static PyTypeObject py_log_destination_type;
+
+static gboolean
+_py_is_log_destination(PyObject *obj)
+{
+  return PyType_IsSubtype(Py_TYPE(obj), &py_log_destination_type);
+}
+
 /** Setters & config glue **/
 
 void
@@ -99,12 +111,6 @@ python_dd_set_loaders(LogDriver *d, GList *loaders)
   self->loaders = loaders;
 }
 
-PyObject *
-python_dd_create_arg_dict(PythonDestDriver *self)
-{
-  return _py_create_arg_dict(self->options);
-}
-
 LogTemplateOptions *
 python_dd_get_template_options(LogDriver *d)
 {
@@ -196,7 +202,7 @@ _py_invoke_open(PythonDestDriver *self)
     {
       if (ret == Py_None)
         {
-          msg_warning_once("Since " VERSION_3_25 ", the return value of open method in python destination "
+          msg_warning_once("python-dest: Since " VERSION_3_25 ", the return value of the open() method "
                            "is used as success/failure indicator. Please use return True or return False explicitly",
                            evt_tag_str("class", self->class));
           result = TRUE;
@@ -226,21 +232,20 @@ _py_invoke_close(PythonDestDriver *self)
 static LogThreadedResult
 _as_int(PyObject *obj)
 {
-  int result = pyobject_as_int(obj);
-  if (result == -1 && PyErr_Occurred())
+  gint64 result;
+  if (!py_long_to_long(obj, &result) && PyErr_Occurred())
     {
       gchar buf[256];
-      _py_format_exception_text(buf, sizeof(buf));
 
-      msg_error("Error converting PyObject to int. Retrying message later",
-                evt_tag_str("exception", buf));
+      msg_error("python-dest: Error converting the result of send() to a LogDestinationResult enum. Retrying message later",
+                evt_tag_str("exception", _py_format_exception_text(buf, sizeof(buf))));
       _py_finish_exception_handling();
       return LTR_ERROR;
     }
 
   if (result < 0 || result >= LTR_MAX)
     {
-      msg_error("Python: worker insert result out of range. Retrying message later",
+      msg_error("python-dest: The result of send() is out of range, please use the LogDestinationResult enum (or a bool) as return value. Retrying message later",
                 evt_tag_int("result", result));
       return LTR_ERROR;
     }
@@ -316,7 +321,7 @@ _py_clear(PyObject *self)
 static void
 _inject_const(PythonDestDriver *self, const gchar *field_name, gint value)
 {
-  PyObject *pyint = int_as_pyobject(value);
+  PyObject *pyint = py_long_from_long(value);
   PyObject_SetAttrString(self->py.class, field_name, pyint);
   g_ptr_array_add(self->py._refs_to_clean, pyint);
 };
@@ -336,51 +341,74 @@ _inject_worker_insert_result_consts(PythonDestDriver *self)
 static PyObject *
 py_get_persist_name(PythonDestDriver *self)
 {
-  return _py_string_from_string(python_dd_format_persist_name(&self->super.super.super.super), -1);
+  return py_string_from_string(python_dd_format_persist_name(&self->super.super.super.super), -1);
 }
 
 static gboolean
 _py_init_bindings(PythonDestDriver *self)
 {
+  GlobalConfig *cfg = log_pipe_get_config(&self->super.super.super.super);
+
   self->py._refs_to_clean = g_ptr_array_new_with_free_func((GDestroyNotify)_py_clear);
 
   self->py.class = _py_resolve_qualified_name(self->class);
   if (!self->py.class)
     {
       gchar buf[256];
-      _py_format_exception_text(buf, sizeof(buf));
 
-      msg_error("Error looking Python driver class",
+      msg_error("python-dest: Error looking up Python driver class",
                 evt_tag_str("driver", self->super.super.super.id),
                 evt_tag_str("class", self->class),
-                evt_tag_str("exception", buf));
+                evt_tag_str("exception", _py_format_exception_text(buf, sizeof(buf))));
       _py_finish_exception_handling();
       return FALSE;
     }
 
   _inject_worker_insert_result_consts(self);
 
-  self->py.log_template_options = py_log_template_options_new(&self->template_options);
-  PyObject_SetAttrString(self->py.class, "template_options", self->py.log_template_options);
-  Py_DECREF(self->py.log_template_options);
-  self->py.seqnum = py_integer_pointer_new(&self->super.worker.instance.seq_num);
-  PyObject_SetAttrString(self->py.class, "seqnum", self->py.seqnum);
-  Py_DECREF(self->py.seqnum);
+  PyObject *py_log_template_options = py_log_template_options_new(&self->template_options, cfg);
+  PyObject_SetAttrString(self->py.class, "template_options", py_log_template_options);
+  Py_DECREF(py_log_template_options);
+
+  PyObject *py_seqnum = py_integer_pointer_new(&self->super.worker.instance.seq_num);
+  PyObject_SetAttrString(self->py.class, "seqnum", py_seqnum);
+  Py_DECREF(py_seqnum);
 
   self->py.instance = _py_invoke_function(self->py.class, NULL, self->class, self->super.super.super.id);
   if (!self->py.instance)
     {
-      gchar buf[256];
-      _py_format_exception_text(buf, sizeof(buf));
+      gchar buf1[256], buf2[256];
 
-      msg_error("Error instantiating Python driver class",
+      msg_error("python-dest: Error instantiating Python driver class",
                 evt_tag_str("driver", self->super.super.super.id),
                 evt_tag_str("class", self->class),
-                evt_tag_str("exception", buf));
+                evt_tag_str("class-repr", _py_object_repr(self->py.class, buf1, sizeof(buf1))),
+                evt_tag_str("exception", _py_format_exception_text(buf2, sizeof(buf2))));
       _py_finish_exception_handling();
       return FALSE;
     }
 
+  if (!_py_is_log_destination(self->py.instance))
+    {
+      gchar buf[256];
+
+      if (!cfg_is_config_version_older(cfg, VERSION_VALUE_4_0))
+        {
+          msg_error("python-dest: Error initializing Python destination, class is not a subclass of LogDestination",
+                    evt_tag_str("driver", self->super.super.super.id),
+                    evt_tag_str("class", self->class),
+                    evt_tag_str("class-repr", _py_object_repr(self->py.class, buf, sizeof(buf))));
+          return FALSE;
+        }
+      msg_warning("WARNING: " VERSION_4_0 " requires that your python() destination class derives "
+                  "from syslogng.LogDestination. Please change the class declaration to explicitly "
+                  "inherit from syslogng.LogDestination. syslog-ng now operates in compatibility mode",
+                  evt_tag_str("driver", self->super.super.super.id),
+                  evt_tag_str("class", self->class),
+                  evt_tag_str("class-repr", _py_object_repr(self->py.class, buf, sizeof(buf))));
+
+    }
+
   /* these are fast paths, store references to be faster */
   self->py.is_opened = _py_get_attr_or_null(self->py.instance, "is_opened");
   self->py.open = _py_get_attr_or_null(self->py.instance, "open");
@@ -389,7 +417,7 @@ _py_init_bindings(PythonDestDriver *self)
   self->py.generate_persist_name = _py_get_attr_or_null(self->py.instance, "generate_persist_name");
   if (!self->py.send)
     {
-      msg_error("Error initializing Python destination, class does not have a send() method",
+      msg_error("python-dest: Error initializing Python destination, class does not have a send() method",
                 evt_tag_str("driver", self->super.super.super.id),
                 evt_tag_str("class", self->class));
       return FALSE;
@@ -405,8 +433,6 @@ _py_init_bindings(PythonDestDriver *self)
   g_ptr_array_add(self->py._refs_to_clean, self->py.open);
   g_ptr_array_add(self->py._refs_to_clean, self->py.flush);
   g_ptr_array_add(self->py._refs_to_clean, self->py.send);
-  g_ptr_array_add(self->py._refs_to_clean, self->py.log_template_options);
-  g_ptr_array_add(self->py._refs_to_clean, self->py.seqnum);
   g_ptr_array_add(self->py._refs_to_clean, self->py.generate_persist_name);
 
   return TRUE;
@@ -423,7 +449,7 @@ _py_init_object(PythonDestDriver *self)
 {
   if (!_py_get_attr_or_null(self->py.instance, "init"))
     {
-      msg_debug("Missing Python method, init()",
+      msg_debug("python-dest: Missing Python method, init()",
                 evt_tag_str("driver", self->super.super.super.id),
                 evt_tag_str("class", self->class));
       return TRUE;
@@ -431,7 +457,7 @@ _py_init_object(PythonDestDriver *self)
 
   if (!_py_invoke_init(self))
     {
-      msg_error("Error initializing Python driver object, init() returned FALSE",
+      msg_error("python-dest: Error initializing Python driver object, init() returned FALSE",
                 evt_tag_str("driver", self->super.super.super.id),
                 evt_tag_str("class", self->class));
       return FALSE;
@@ -442,19 +468,20 @@ _py_init_object(PythonDestDriver *self)
 static gboolean
 _py_construct_message(PythonDestDriver *self, LogMessage *msg, PyObject **msg_object)
 {
+  GlobalConfig *cfg = log_pipe_get_config(&self->super.super.super.super);
   gboolean success;
   *msg_object = NULL;
 
   if (self->vp)
     {
-      LogTemplateEvalOptions options = {&self->template_options, LTZ_LOCAL, self->super.worker.instance.seq_num, NULL};
+      LogTemplateEvalOptions options = {&self->template_options, LTZ_LOCAL, self->super.worker.instance.seq_num, NULL, LM_VT_STRING};
       success = py_value_pairs_apply(self->vp, &options, msg, msg_object);
       if (!success && (self->template_options.on_error & ON_ERROR_DROP_MESSAGE))
         return FALSE;
     }
   else
     {
-      *msg_object = py_log_message_new(msg);
+      *msg_object = py_log_message_new(msg, cfg);
     }
 
   return TRUE;
@@ -557,27 +584,29 @@ python_dd_init(LogPipe *d)
 
   if (!self->class)
     {
-      msg_error("Error initializing Python destination: no script specified!",
+      msg_error("python-dest: Error initializing Python destination, the class() option is not specified",
                 evt_tag_str("driver", self->super.super.super.id));
       return FALSE;
     }
 
-  if (!log_threaded_dest_driver_init_method(d))
-    return FALSE;
-
   log_template_options_init(&self->template_options, cfg);
   self->super.time_reopen = 1;
 
   gstate = PyGILState_Ensure();
-
   _py_perform_imports(self->loaders);
-  if (!_py_init_bindings(self) ||
-      !_py_init_object(self))
+  if (!_py_init_bindings(self))
     goto fail;
-
   PyGILState_Release(gstate);
 
-  msg_verbose("Python destination initialized",
+  if (!log_threaded_dest_driver_init_method(d))
+    return FALSE;
+
+  gstate = PyGILState_Ensure();
+  if (!_py_init_object(self))
+    goto fail;
+  PyGILState_Release(gstate);
+
+  msg_verbose("python-dest: Python destination initialized",
               evt_tag_str("driver", self->super.super.super.id),
               evt_tag_str("class", self->class));
 
@@ -650,3 +679,33 @@ python_dd_new(GlobalConfig *cfg)
 
   return (LogDriver *)self;
 }
+
+static PyTypeObject py_log_destination_type =
+{
+  PyVarObject_HEAD_INIT(&PyType_Type, 0)
+  .tp_name = "LogDestination",
+  .tp_basicsize = sizeof(PyLogDestination),
+  .tp_dealloc = py_slng_generic_dealloc,
+  .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
+  .tp_doc = "The LogDestination class is a base class for custom Python sources.",
+  .tp_new = PyType_GenericNew,
+  0,
+};
+
+void
+py_log_destination_global_init(void)
+{
+  PyObject *module = PyImport_AddModule("_syslogng");
+  PyObject *enum_seq = PyList_New(6);
+
+  PyList_SetItem(enum_seq, 0, Py_BuildValue("(si)", "DROP", LTR_DROP));
+  PyList_SetItem(enum_seq, 1, Py_BuildValue("(si)", "ERROR", LTR_ERROR));
+  PyList_SetItem(enum_seq, 2, Py_BuildValue("(si)", "EXPLICIT_ACK_MGMT", LTR_EXPLICIT_ACK_MGMT));
+  PyList_SetItem(enum_seq, 3, Py_BuildValue("(si)", "SUCCESS", LTR_SUCCESS));
+  PyList_SetItem(enum_seq, 4, Py_BuildValue("(si)", "QUEUED", LTR_QUEUED));
+  PyList_SetItem(enum_seq, 5, Py_BuildValue("(si)", "NOT_CONNECTED", LTR_NOT_CONNECTED));
+  PyModule_AddObject(module, "LogDestinationResult", _py_construct_enum("LogDestinationResult", enum_seq));
+
+  PyType_Ready(&py_log_destination_type);
+  PyModule_AddObject(module, "LogDestination", (PyObject *) &py_log_destination_type);
+}
