@@ -22,18 +22,25 @@
  *
  */
 #include "python-helpers.h"
+#include "python-types.h"
 #include "scratch-buffers.h"
 #include "str-utils.h"
 #include "messages.h"
+#include "reloc.h"
 
 const gchar *
 _py_get_callable_name(PyObject *callable, gchar *buf, gsize buf_len)
 {
+  PyObject *exc, *value, *tb;
+
+  PyErr_Fetch(&exc, &value, &tb);
+
   PyObject *name = PyObject_GetAttrString(callable, "__name__");
 
-  if (name && _py_is_string(name))
+  const gchar *str;
+  if (name && py_bytes_or_string_to_string(name, &str))
     {
-      g_strlcpy(buf, _py_get_string_as_string(name), buf_len);
+      g_strlcpy(buf, str, buf_len);
     }
   else
     {
@@ -41,6 +48,8 @@ _py_get_callable_name(PyObject *callable, gchar *buf, gsize buf_len)
       g_strlcpy(buf, "<unknown>", buf_len);
     }
   Py_XDECREF(name);
+
+  PyErr_Restore(exc, value, tb);
   return buf;
 }
 
@@ -98,9 +107,10 @@ _py_format_exception_text(gchar *buf, gsize buf_len)
   if (!str)
     PyErr_Clear();
 
-  if (str && _py_is_string(str))
+  const gchar *str_as_c_str;
+  if (str && py_bytes_or_string_to_string(str, &str_as_c_str))
     {
-      g_snprintf(buf, buf_len, "%s: %s", ((PyTypeObject *) exc)->tp_name, _py_get_string_as_string(str));
+      g_snprintf(buf, buf_len, "%s: %s", ((PyTypeObject *) exc)->tp_name, str_as_c_str);
     }
   else
     {
@@ -150,7 +160,23 @@ _py_do_import(const gchar *modname)
 
   modobj = PyImport_Import(module);
   Py_DECREF(module);
-  if (!modobj)
+  if (modobj)
+    {
+      PyObject  *mod_filename = PyModule_GetFilenameObject(modobj);
+
+      if (!mod_filename)
+        {
+          /* this exception only means that this module does not have a
+           * __name__ attribute */
+          PyErr_Clear();
+        }
+
+      msg_debug("python: importing Python module",
+                evt_tag_str("module", modname),
+                evt_tag_str("filename", mod_filename ? PyUnicode_AsUTF8(mod_filename) : "unknown"));
+      Py_XDECREF(mod_filename);
+    }
+  else
     {
       gchar buf[256];
 
@@ -208,9 +234,11 @@ exit:
 static void
 _insert_to_dict(gpointer key, gpointer value, gpointer dict)
 {
-  PyObject *key_pyobj = _py_string_from_string((gchar *) key, -1);
-  PyObject *value_pyobj = _py_string_from_string((gchar *) value, -1);
+  PyObject *key_pyobj = py_string_from_string((gchar *) key, -1);
+  PyObject *value_pyobj = py_string_from_string((gchar *) value, -1);
   PyDict_SetItem( (PyObject *) dict, key_pyobj, value_pyobj);
+  Py_XDECREF(key_pyobj);
+  Py_XDECREF(value_pyobj);
 }
 
 PyObject *
@@ -221,6 +249,14 @@ _py_create_arg_dict(GHashTable *args)
   return arg_dict;
 }
 
+PyObject *
+_py_construct_cfg_args(CfgArgs *args)
+{
+  PyObject *arg_dict = PyDict_New();
+  cfg_args_foreach(args, _insert_to_dict, arg_dict);
+  return arg_dict;
+}
+
 PyObject *
 _py_invoke_function(PyObject *func, PyObject *arg, const gchar *class, const gchar *caller_context)
 {
@@ -233,7 +269,7 @@ _py_invoke_function(PyObject *func, PyObject *arg, const gchar *class, const gch
 
       msg_error("Exception while calling a Python function",
                 evt_tag_str("caller", caller_context),
-                evt_tag_str("class", class),
+                evt_tag_str("class", class ? : "unknown"),
                 evt_tag_str("function", _py_get_callable_name(func, buf1, sizeof(buf1))),
                 evt_tag_str("exception", _py_format_exception_text(buf2, sizeof(buf2))));
       _py_finish_exception_handling();
@@ -364,91 +400,45 @@ _py_perform_imports(GList *imports)
   g_list_foreach(imports, _foreach_import, NULL);
 }
 
-gboolean
-_py_is_string(PyObject *object)
-{
-  return PyBytes_Check(object) || PyUnicode_Check(object);
-}
-
-
-/* NOTE: this function returns a managed memory area pointing to an utf8
- * representation of the string, with the following constraints:
- *
- *   1) we basically assume that non-unicode strings (both in Python2 and
- *   Python3) are in utf8 or at least utf8 compatible (e.g.  ascii).  It
- *   doesn't really make sense otherwise.  If we don't the resulting string
- *   is not going to be utf8, rather it would be the system codepage.
- *
- *   2) in the case of Python3 we are using the utf8 cache in the unicode
- *   instance.  In the case of Python2 we are allocating a scratch buffer to
- *   hold the data for us.
- **/
-
 const gchar *
-_py_get_string_as_string(PyObject *object)
+_py_object_repr(PyObject *s, gchar *buf, gsize buflen)
 {
-  if (PyBytes_Check(object))
-    return PyBytes_AsString(object);
-#if PY_MAJOR_VERSION >= 3
-  else if (PyUnicode_Check(object))
-    return PyUnicode_AsUTF8(object);
-#elif PY_MAJOR_VERSION < 3
-  else if (PyUnicode_Check(object))
+  const gchar *res;
+
+  PyObject *r = PyObject_Repr(s);
+  if (!r)
     {
-      PyObject *utf8_bytes = PyUnicode_AsUTF8String(object);
-      GString *buffer = scratch_buffers_alloc();
-      g_string_assign_len(buffer, PyBytes_AsString(utf8_bytes), PyBytes_Size(utf8_bytes));
-      Py_XDECREF(utf8_bytes);
-      return buffer->str;
+      _py_finish_exception_handling();
+      g_strlcpy(buf, "<unknown object>", buflen);
+      return buf;
     }
-#endif
-  g_assert_not_reached();
+  if (py_bytes_or_string_to_string(r, &res))
+    g_strlcpy(buf, res, buflen);
+  Py_XDECREF(r);
+  return buf;
 }
 
 PyObject *
-_py_string_from_string(const gchar *str, gssize len)
+_py_construct_enum(const gchar *name, PyObject *sequence)
 {
-#if PY_MAJOR_VERSION >= 3
-  const gchar *charset;
+  PyObject *enum_module = PyImport_ImportModule("enum");
 
-  /* NOTE: g_get_charset() returns if the current character set is utf8 */
-  if (g_get_charset(&charset))
-    {
-      if (len < 0)
-        return PyUnicode_FromString(str);
-      else
-        return PyUnicode_FromStringAndSize(str, len);
-    }
-  else
-    {
-      GError *error = NULL;
-      gsize bytes_read, bytes_written;
-      gchar *utf8_string;
-      PyObject *res;
+  if (!enum_module)
+    return NULL;
 
-      utf8_string = g_locale_to_utf8(str, len, &bytes_read, &bytes_written, &error);
-      if (utf8_string)
-        {
-          res = PyUnicode_FromStringAndSize(utf8_string, bytes_written);
-          g_free(utf8_string);
-          return res;
-        }
-      else
-        {
-          if (len >= 0)
-            return PyBytes_FromStringAndSize(str, len);
-          else
-            return PyBytes_FromString(str);
-        }
-    }
-#elif PY_MAJOR_VERSION < 3
-  if (len >= 0)
-    return PyBytes_FromStringAndSize(str, len);
-  else
-    return PyBytes_FromString(str);
-#endif
+  PyObject *enum_dict = PyModule_GetDict(enum_module);
+  PyObject *enum_new = PyDict_GetItemString(enum_dict, "IntEnum");
+
+  if (!enum_new)
+    return NULL;
+
+  PyObject *result = PyObject_CallFunction(enum_new, "sO", name, sequence);
+  Py_XDECREF(enum_module);
+
+  return result;
 }
 
+
 void
 py_slng_generic_dealloc(PyObject *self)
 {
