@@ -25,14 +25,16 @@
 #define PYTHON_HELPERS_H_INCLUDED 1
 
 #include "python-module.h"
+#include "cfg-args.h"
 
-void _py_get_callable_name(PyObject *callable, gchar *buf, gsize buf_len);
-void _py_format_exception_text(gchar *buf, gsize buf_len);
+const gchar *_py_get_callable_name(PyObject *callable, gchar *buf, gsize buf_len);
+const gchar *_py_format_exception_text(gchar *buf, gsize buf_len);
 void _py_finish_exception_handling(void);
 PyObject *_py_get_attr_or_null(PyObject *o, const gchar *attr);
 PyObject *_py_do_import(const gchar *modname);
 PyObject *_py_resolve_qualified_name(const gchar *name);
 PyObject *_py_create_arg_dict(GHashTable *args);
+PyObject *_py_construct_cfg_args(CfgArgs *args);
 PyObject *_py_invoke_function(PyObject *func, PyObject *arg, const gchar *class, const gchar *caller_context);
 PyObject *_py_invoke_function_with_args(PyObject *func, PyObject *args, const gchar *class,
                                         const gchar *caller_context);
@@ -48,9 +50,9 @@ gboolean _py_invoke_bool_method_by_name_with_args(PyObject *instance, const gcha
 gboolean _py_invoke_bool_method_by_name(PyObject *instance, const gchar *method_name, const gchar *class,
                                         const gchar *module);
 void _py_perform_imports(GList *imports);
-gboolean _py_is_string(PyObject *object);
-const gchar *_py_get_string_as_string(PyObject *object);
-PyObject *_py_string_from_string(const gchar *str, gssize len);
+const gchar *_py_object_repr(PyObject *s, gchar *buf, gsize buflen);
+PyObject *_py_construct_enum(const gchar *name, PyObject *sequence);
 
 void py_slng_generic_dealloc(PyObject *self);
+
 #endif
