@@ -24,12 +24,17 @@
 #include "python-logmsg.h"
 #include "compat/compat-python.h"
 #include "python-helpers.h"
+#include "python-types.h"
+#include "python-main.h"
 #include "logmsg/logmsg.h"
 #include "messages.h"
 #include "timeutils/cache.h"
 #include "str-utils.h"
 #include "timeutils/unixtime.h"
 #include "timeutils/misc.h"
+#include "msg-format.h"
+#include "scratch-buffers.h"
+#include "cfg.h"
 
 #include <datetime.h>
 
@@ -39,58 +44,21 @@ py_is_log_message(PyObject *obj)
   return PyType_IsSubtype(Py_TYPE(obj), &py_log_message_type);
 }
 
-static int
-_str_cmp(const void *s1, const void *s2)
-{
-  return strcmp(*(const gchar **)s1, *(const gchar **)s2);
-}
-
-static void
-_populate_blacklisted_keys(const gchar ***blacklist, size_t *n)
-{
-  static const gchar *keys[] =
-  {
-    "S_STAMP", "_", "C_STAMP"
-  };
-  static gboolean keys_sorted = FALSE;
-  if (!keys_sorted)
-    {
-      keys_sorted = TRUE;
-      qsort(&keys[0], sizeof(keys)/sizeof(keys[0]), sizeof(gchar *), _str_cmp);
-    }
-  *blacklist = keys;
-  *n = sizeof(keys)/sizeof(keys[0]);
-}
-
-static gboolean
-_is_key_blacklisted(const gchar *key)
-{
-  const gchar **blacklist = NULL;
-  size_t n = 0;
-  _populate_blacklisted_keys(&blacklist, &n);
-  return (bsearch(&key, blacklist, n, sizeof(gchar *), _str_cmp) != NULL);
-}
-
 static PyObject *
 _py_log_message_subscript(PyObject *o, PyObject *key)
 {
-  if (!_py_is_string(key))
+  const gchar *name;
+  if (!py_bytes_or_string_to_string(key, &name))
     {
       PyErr_SetString(PyExc_TypeError, "key is not a string object");
       return NULL;
     }
 
-  const gchar *name = _py_get_string_as_string(key);
-
-  if (_is_key_blacklisted(name))
-    {
-      PyErr_Format(PyExc_KeyError, "Blacklisted attribute %s was requested", name);
-      return NULL;
-    }
   NVHandle handle = log_msg_get_value_handle(name);
   PyLogMessage *py_msg = (PyLogMessage *)o;
   gssize value_len = 0;
-  const gchar *value = log_msg_get_value(py_msg->msg, handle, &value_len);
+  LogMessageValueType type;
+  const gchar *value = log_msg_get_value_with_type(py_msg->msg, handle, &value_len, &type);
 
   if (!value)
     {
@@ -98,15 +66,19 @@ _py_log_message_subscript(PyObject *o, PyObject *key)
       return NULL;
     }
 
+  if (py_msg->cast_to_strings)
+    type = LM_VT_STRING;
+
   APPEND_ZERO(value, value, value_len);
 
-  return PyBytes_FromString(value);
+  return py_obj_from_log_msg_value(value, value_len, type);
 }
 
 static int
 _py_log_message_ass_subscript(PyObject *o, PyObject *key, PyObject *value)
 {
-  if (!_py_is_string(key))
+  const gchar *name;
+  if (!py_bytes_or_string_to_string(key, &name))
     {
       PyErr_SetString(PyExc_TypeError, "key is not a string object");
       return -1;
@@ -114,7 +86,6 @@ _py_log_message_ass_subscript(PyObject *o, PyObject *key, PyObject *value)
 
   PyLogMessage *py_msg = (PyLogMessage *) o;
   LogMessage *msg = py_msg->msg;
-  const gchar *name = _py_get_string_as_string(key);
 
   if (log_msg_is_write_protected(msg))
     {
@@ -127,20 +98,33 @@ _py_log_message_ass_subscript(PyObject *o, PyObject *key, PyObject *value)
 
   NVHandle handle = log_msg_get_value_handle(name);
 
-  if (value && _py_is_string(value))
-    {
-      log_msg_set_value(py_msg->msg, handle, _py_get_string_as_string(value), -1);
-    }
-  else
+  if (!value)
+    return -1;
+
+  if (py_msg->cast_to_strings && !is_py_obj_bytes_or_string_type(value))
     {
       PyErr_Format(PyExc_ValueError,
                    "str or unicode object expected as log message values, got type %s (key %s). "
                    "Earlier syslog-ng accepted any type, implicitly converting it to a string. "
+                   "Later syslog-ng (at least 4.0) will store the value with the correct type. "
                    "With this version please convert it explicitly to a string using str()",
                    value->ob_type->tp_name, name);
       return -1;
     }
 
+  ScratchBuffersMarker marker;
+  GString *log_msg_value = scratch_buffers_alloc_and_mark(&marker);
+  LogMessageValueType type;
+
+  if (!py_obj_to_log_msg_value(value, log_msg_value, &type))
+    {
+      scratch_buffers_reclaim_marked(marker);
+      return -1;
+    }
+
+  log_msg_set_value_with_type(py_msg->msg, handle, log_msg_value->str, -1, type);
+
+  scratch_buffers_reclaim_marked(marker);
   return 0;
 }
 
@@ -153,7 +137,7 @@ py_log_message_free(PyLogMessage *self)
 }
 
 PyObject *
-py_log_message_new(LogMessage *msg)
+py_log_message_new(LogMessage *msg, GlobalConfig *cfg)
 {
   PyLogMessage *self;
 
@@ -163,23 +147,25 @@ py_log_message_new(LogMessage *msg)
 
   self->msg = log_msg_ref(msg);
   self->bookmark_data = NULL;
+
+  if (!cfg_is_typing_feature_enabled(cfg))
+    self->cast_to_strings = TRUE;
+  else
+    self->cast_to_strings = FALSE;
   return (PyObject *) self;
 }
 
-static PyObject *
-py_log_message_new_empty(PyTypeObject *subtype, PyObject *args, PyObject *kwds)
+static int
+py_log_message_init(PyObject *s, PyObject *args, PyObject *kwds)
 {
+  PyLogMessage *self = (PyLogMessage *) s;
   PyObject *bookmark_data = NULL;
   const gchar *message = NULL;
-  gint message_length = 0;
+  Py_ssize_t message_length = 0;
 
   static const gchar *kwlist[] = {"message", "bookmark", NULL};
   if (!PyArg_ParseTupleAndKeywords(args, kwds, "|z#O", (gchar **) kwlist, &message, &message_length, &bookmark_data))
-    return NULL;
-
-  PyLogMessage *self = (PyLogMessage *) subtype->tp_alloc(subtype, 0);
-  if (!self)
-    return NULL;
+    return -1;
 
   self->msg = log_msg_new_empty();
   self->bookmark_data = NULL;
@@ -191,7 +177,7 @@ py_log_message_new_empty(PyTypeObject *subtype, PyObject *args, PyObject *kwds)
   Py_XINCREF(bookmark_data);
   self->bookmark_data = bookmark_data;
 
-  return (PyObject *) self;
+  return 0;
 }
 
 static PyMappingMethods py_log_message_mapping =
@@ -203,7 +189,7 @@ static PyMappingMethods py_log_message_mapping =
 
 static gboolean
 _collect_nvpair_names_from_logmsg(NVHandle handle, const gchar *name, const gchar *value, gssize value_len,
-                                  gpointer user_data)
+                                  LogMessageValueType type, gpointer user_data)
 {
   PyObject *list = (PyObject *)user_data;
 
@@ -217,7 +203,7 @@ _collect_nvpair_names_from_logmsg(NVHandle handle, const gchar *name, const gcha
 static gboolean
 _is_macro_name_visible_to_user(const gchar *name, NVHandle handle)
 {
-  return log_msg_is_handle_macro(handle) && !_is_key_blacklisted(name);
+  return log_msg_is_handle_macro(handle);
 }
 
 static void
@@ -262,85 +248,11 @@ py_log_message_set_pri(PyLogMessage *self, PyObject *args, PyObject *kwrds)
 }
 
 static PyObject *
-_datetime_timestamp(PyObject *py_datetime)
+py_log_message_get_pri(PyLogMessage *self, PyObject *args, PyObject *kwrds)
 {
-  PyObject *py_tzinfo = _py_get_attr_or_null(py_datetime, "tzinfo");
-  if (!py_tzinfo)
-    {
-      PyErr_Format(PyExc_ValueError, "Error obtaining tzinfo");
-      return NULL;
-    }
-  PyObject *py_epoch = PyDateTimeAPI->DateTime_FromDateAndTime(1970, 1, 1, 0, 0, 0, 0, py_tzinfo,
-                       PyDateTimeAPI->DateTimeType);
-
-  PyObject *py_delta = _py_invoke_method_by_name(py_datetime, "__sub__", py_epoch,
-                                                 "PyDateTime", "py_datetime_to_logstamp");
-  if (!py_delta)
-    {
-      Py_XDECREF(py_tzinfo);
-      Py_XDECREF(py_epoch);
-      PyErr_Format(PyExc_ValueError, "Error calculating POSIX timestamp");
-      return NULL;
-    }
-
-  PyObject *py_posix_timestamp = _py_invoke_method_by_name(py_delta, "total_seconds", NULL,
-                                                           "PyDateTime", "py_datetime_to_logstamp");
-  Py_XDECREF(py_tzinfo);
-  Py_XDECREF(py_delta);
-  Py_XDECREF(py_epoch);
-
-  return py_posix_timestamp;
-}
-
-static gboolean
-_datetime_get_gmtoff(PyObject *py_datetime, gint *utcoffset)
-{
-  *utcoffset = -1;
-  PyObject *py_utcoffset = _py_invoke_method_by_name(py_datetime, "utcoffset", NULL, "PyDateTime",
-                                                     "py_datetime_to_logstamp");
-  if (!py_utcoffset)
-    {
-      PyErr_Format(PyExc_ValueError, "Error obtaining timezone info");
-      return FALSE;
-    }
-
-  if (py_utcoffset != Py_None)
-    {
-      *utcoffset = PyDateTime_DELTA_GET_SECONDS(py_utcoffset);
-    }
-
-  Py_XDECREF(py_utcoffset);
-  return TRUE;
-}
-
-static gboolean
-py_datetime_to_logstamp(PyObject *py_timestamp, UnixTime *logstamp)
-{
-  if (!PyDateTime_Check(py_timestamp))
-    {
-      PyErr_Format(PyExc_TypeError, "datetime expected in the first parameter");
-      return FALSE;
-    }
-
-  PyObject *py_posix_timestamp = _datetime_timestamp(py_timestamp);
-  if (!py_posix_timestamp)
-    return FALSE;
-
-  gdouble posix_timestamp = PyFloat_AsDouble(py_posix_timestamp);
-
-  Py_XDECREF(py_posix_timestamp);
-
-  gint local_gmtoff;
-  if (!_datetime_get_gmtoff(py_timestamp, &local_gmtoff))
-    return FALSE;
-  if (local_gmtoff == -1)
-    local_gmtoff = get_local_timezone_ofs(posix_timestamp);
-
-  logstamp->ut_sec = (time_t) posix_timestamp;
-  logstamp->ut_usec = posix_timestamp * 10e5 - logstamp->ut_sec * 10e5;
-  logstamp->ut_gmtoff = local_gmtoff;
-
-  return TRUE;
+  if (!PyArg_ParseTuple(args, ""))
+    return NULL;
+  return PyLong_FromLong(self->msg->pri);
 }
 
 static PyObject *
@@ -352,12 +264,24 @@ py_log_message_set_timestamp(PyLogMessage *self, PyObject *args, PyObject *kwrds
   if (!PyArg_ParseTupleAndKeywords(args, kwrds, "O", (gchar **) kwlist, &py_timestamp))
     return NULL;
 
-  if (!py_datetime_to_logstamp((PyObject *) py_timestamp, &self->msg->timestamps[LM_TS_STAMP]))
-    return NULL;
+  if (!py_datetime_to_unix_time((PyObject *) py_timestamp, &self->msg->timestamps[LM_TS_STAMP]))
+    {
+      PyErr_Format(PyExc_ValueError, "Error extracting timestamp from value, expected a datetime instance");
+      return NULL;
+    }
 
   Py_RETURN_NONE;
 }
 
+static PyObject *
+py_log_message_get_timestamp(PyLogMessage *self, PyObject *args, PyObject *kwrds)
+{
+  if (!PyArg_ParseTuple(args, ""))
+    return NULL;
+
+  return py_datetime_from_unix_time(&self->msg->timestamps[LM_TS_STAMP]);
+}
+
 static PyObject *
 py_log_message_set_bookmark(PyLogMessage *self, PyObject *args, PyObject *kwrds)
 {
@@ -379,7 +303,7 @@ static PyObject *
 py_log_message_parse(PyObject *_none, PyObject *args, PyObject *kwrds)
 {
   const gchar *raw_msg;
-  gint raw_msg_length;
+  Py_ssize_t raw_msg_length;
 
   PyObject *py_parse_options;
 
@@ -408,7 +332,7 @@ py_log_message_parse(PyObject *_none, PyObject *args, PyObject *kwrds)
       return NULL;
     }
 
-  py_msg->msg = log_msg_new(raw_msg, raw_msg_length, parse_options);
+  py_msg->msg = msg_format_parse(parse_options, (const guchar *) raw_msg, raw_msg_length);
   py_msg->bookmark_data = NULL;
 
   return (PyObject *) py_msg;
@@ -417,8 +341,10 @@ py_log_message_parse(PyObject *_none, PyObject *args, PyObject *kwrds)
 static PyMethodDef py_log_message_methods[] =
 {
   { "keys", (PyCFunction)_logmessage_get_keys_method, METH_NOARGS, "Return keys." },
-  { "set_pri", (PyCFunction)py_log_message_set_pri, METH_VARARGS | METH_KEYWORDS, "Set priority" },
+  { "set_pri", (PyCFunction)py_log_message_set_pri, METH_VARARGS | METH_KEYWORDS, "Set syslog priority" },
+  { "get_pri", (PyCFunction)py_log_message_get_pri, METH_VARARGS | METH_KEYWORDS, "Get syslog priority" },
   { "set_timestamp", (PyCFunction)py_log_message_set_timestamp, METH_VARARGS | METH_KEYWORDS, "Set timestamp" },
+  { "get_timestamp", (PyCFunction)py_log_message_get_timestamp, METH_VARARGS | METH_KEYWORDS, "Get timestamp" },
   { "set_bookmark", (PyCFunction)py_log_message_set_bookmark, METH_VARARGS | METH_KEYWORDS, "Set bookmark" },
   { "parse", (PyCFunction)py_log_message_parse, METH_STATIC|METH_VARARGS|METH_KEYWORDS, "Parse and create LogMessage" },
   {NULL}
@@ -432,14 +358,15 @@ PyTypeObject py_log_message_type =
   .tp_dealloc = (destructor) py_log_message_free,
   .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
   .tp_doc = "LogMessage class encapsulating a syslog-ng log message",
-  .tp_new = py_log_message_new_empty,
+  .tp_new = PyType_GenericNew,
+  .tp_init = py_log_message_init,
   .tp_as_mapping = &py_log_message_mapping,
   .tp_methods = py_log_message_methods,
   0,
 };
 
 void
-py_log_message_init(void)
+py_log_message_global_init(void)
 {
   PyDateTime_IMPORT;
   PyType_Ready(&py_log_message_type);
