@@ -23,58 +23,53 @@
 #include "python-module.h"
 
 #include <criterion/criterion.h>
+#include <criterion/parameterized.h>
 #include "libtest/msg_parse_lib.h"
 
 #include "python-helpers.h"
+#include "python-types.h"
 #include "python-logmsg.h"
+#include "python-main.h"
 #include "apphook.h"
 #include "logmsg/logmsg.h"
+#include "scratch-buffers.h"
 
 static PyObject *_python_main;
 static PyObject *_python_main_dict;
 
 MsgFormatOptions parse_options;
 
-static void
-_py_init_interpreter(void)
-{
-  py_setup_python_home();
-  Py_Initialize();
-  py_init_argv();
-
-  py_init_threads();
-  py_log_message_init();
-  PyEval_SaveThread();
-}
-
 static void
 _init_python_main(void)
 {
   PyGILState_STATE gstate = PyGILState_Ensure();
   {
-    _python_main = PyImport_AddModule("__main__");
+    PythonConfig *pc = python_config_get(configuration);
+    _python_main = _py_get_main_module(pc);
     _python_main_dict = PyModule_GetDict(_python_main);
+    g_assert(PyRun_String("import datetime", Py_file_input, _python_main_dict, _python_main_dict));
   }
   PyGILState_Release(gstate);
 }
 
-static gchar *
-_dict_clone_value(PyObject *dict, const gchar *key)
+static void
+_assert_python_variable_value(const gchar *variable_name, const gchar *expected_value)
 {
-  PyObject *res_obj = PyDict_GetItemString(dict, key);
-
-  if (!res_obj)
-    return NULL;
-
-  if (!_py_is_string(res_obj))
+  gchar *script = g_strdup_printf("assert %s == %s", variable_name, expected_value);
+  if (!PyRun_String(script, Py_file_input, _python_main_dict, _python_main_dict))
     {
-      Py_XDECREF(res_obj);
-      return NULL;
+      gchar buf[256];
+
+      msg_error("Error in _assert_python_variable_value()",
+                evt_tag_str("script", script),
+                evt_tag_str("exception", _py_format_exception_text(buf, sizeof(buf))));
+      _py_finish_exception_handling();
+
+      g_free(script);
+      cr_assert(FALSE);
     }
 
-  gchar *res = g_strdup(_py_get_string_as_string(res_obj));
-  Py_XDECREF(res_obj);
-  return res;
+  g_free(script);
 }
 
 static PyLogMessage *
@@ -97,43 +92,177 @@ _construct_py_parse_options(void)
   return py_parse_options;
 }
 
-void setup(void)
+void
+setup(void)
 {
+  setenv("TZ", "UTC", TRUE);
   app_startup();
 
   init_parse_options_and_load_syslogformat(&parse_options);
 
-  _py_init_interpreter();
+  _py_init_interpreter(FALSE);
   _init_python_main();
 }
 
-void teardown(void)
+void
+teardown(void)
 {
+  scratch_buffers_explicit_gc();
   deinit_syslogformat_module();
   app_shutdown();
 }
 
 TestSuite(python_log_message, .init = setup, .fini = teardown);
 
-Test(python_log_message, test_python_logmessage_set_value)
+typedef struct _PyLogMessageSetValueTestParams
+{
+  const gchar *py_value_to_set;
+  const gchar *py_value_to_check;
+  const gchar *expected_log_msg_value;
+  LogMessageValueType expected_log_msg_type;
+} PyLogMessageSetValueTestParams;
+
+ParameterizedTestParameters(python_log_message, test_python_logmessage_set_value)
+{
+  static PyLogMessageSetValueTestParams test_data_list[] =
+  {
+    {
+      .py_value_to_set = "'almafa'",
+      .py_value_to_check = "b'almafa'",
+      .expected_log_msg_value = "almafa",
+      .expected_log_msg_type = LM_VT_STRING
+    },
+    {
+      .py_value_to_set = "b'kortefa'",
+      .py_value_to_check = "b'kortefa'",
+      .expected_log_msg_value = "kortefa",
+      .expected_log_msg_type = LM_VT_STRING
+    },
+    {
+      .py_value_to_set = "42",
+      .py_value_to_check = "42",
+      .expected_log_msg_value = "42",
+      .expected_log_msg_type = LM_VT_INTEGER
+    },
+    {
+      .py_value_to_set = "6.9",
+      .py_value_to_check = "6.9",
+      .expected_log_msg_value = "6.900000",
+      .expected_log_msg_type = LM_VT_DOUBLE
+    },
+    {
+      .py_value_to_set = "True",
+      .py_value_to_check = "True",
+      .expected_log_msg_value = "true",
+      .expected_log_msg_type = LM_VT_BOOLEAN
+    },
+    {
+      .py_value_to_set = "None",
+      .py_value_to_check = "None",
+      .expected_log_msg_value = "",
+      .expected_log_msg_type = LM_VT_NULL
+    },
+    {
+      .py_value_to_set = "['a,', ' b', b'c']",
+      .py_value_to_check = "[b'a,', b' b', b'c']",
+      .expected_log_msg_value = "\"a,\",\" b\",c",
+      .expected_log_msg_type = LM_VT_LIST
+    },
+    {
+      .py_value_to_set = "datetime.datetime(2022, 10, 4, 14, 48, 12, 123000)",
+      .py_value_to_check = "datetime.datetime(2022, 10, 4, 14, 48, 12, 123000)",
+      .expected_log_msg_value = "1664894892.123",
+      .expected_log_msg_type = LM_VT_DATETIME
+    },
+  };
+
+  return cr_make_param_array(PyLogMessageSetValueTestParams, test_data_list, G_N_ELEMENTS(test_data_list));
+}
+
+ParameterizedTest(PyLogMessageSetValueTestParams *params, python_log_message, test_python_logmessage_set_value)
 {
   LogMessage *msg = log_msg_new_empty();
 
   PyGILState_STATE gstate;
   gstate = PyGILState_Ensure();
   {
-    PyObject *msg_object = py_log_message_new(msg);
+    cfg_set_version_without_validation(configuration, VERSION_VALUE_4_0);
+    PyObject *msg_object = py_log_message_new(msg, configuration);
 
     PyDict_SetItemString(_python_main_dict, "test_msg", msg_object);
-    const gchar *set_value = "test_msg['test_field'] = 'test_value'\nresult=test_msg['test_field']";
-    PyRun_String(set_value, Py_file_input, _python_main_dict, _python_main_dict);
 
-    gchar *res = _dict_clone_value(_python_main_dict, "result");
-    cr_assert_str_eq(res, "test_value");
-    g_free(res);
+    gchar *script = g_strdup_printf(
+                      "test_msg['test_field'] = %s\n"
+                      "result = test_msg['test_field']\n",
+                      params->py_value_to_set
+                    );
+    if (!PyRun_String(script, Py_file_input, _python_main_dict, _python_main_dict))
+      {
+        PyErr_Print();
+        cr_assert(FALSE, "Error running Python script >>>%s<<<", script);
+      }
 
-    const gchar *test_value=log_msg_get_value_by_name(msg, "test_field", NULL);
-    cr_assert_str_eq(test_value, "test_value");
+    g_free(script);
+
+    _assert_python_variable_value("result", params->py_value_to_check);
+
+    LogMessageValueType type;
+    const gchar *value = log_msg_get_value_by_name_with_type(msg, "test_field", NULL, &type);
+    cr_assert_str_eq(value, params->expected_log_msg_value);
+    cr_assert(type == params->expected_log_msg_type);
+
+    Py_XDECREF(msg_object);
+  }
+  PyGILState_Release(gstate);
+}
+
+Test(python_log_message, test_python_logmessage_set_value_no_typing_support)
+{
+  LogMessage *msg = log_msg_new_empty();
+
+  PyGILState_STATE gstate;
+  gstate = PyGILState_Ensure();
+  {
+    PyObject *msg_object = py_log_message_new(msg, configuration);
+    ((PyLogMessage *) msg_object)->cast_to_strings = TRUE;
+
+    PyDict_SetItemString(_python_main_dict, "test_msg", msg_object);
+
+    const gchar *script = "test_msg['test_field'] = 42\n";
+    cr_assert_not(PyRun_String(script, Py_file_input, _python_main_dict, _python_main_dict));
+
+    Py_XDECREF(msg_object);
+  }
+  PyGILState_Release(gstate);
+}
+
+Test(python_log_message, test_python_logmessage_get_value_no_typing_support)
+{
+  const gchar *test_key = "test_field";
+  const gchar *test_value = "42";
+
+  LogMessage *msg = log_msg_new_empty();
+
+  // set from C code as INTEGER
+  log_msg_set_value_by_name_with_type(msg, test_key, test_value, strlen(test_value), LM_VT_INTEGER);
+
+  PyGILState_STATE gstate;
+  gstate = PyGILState_Ensure();
+  {
+    PyObject *msg_object = py_log_message_new(msg, configuration);
+    ((PyLogMessage *) msg_object)->cast_to_strings = TRUE;
+
+    PyDict_SetItemString(_python_main_dict, "test_msg", msg_object);
+
+    // in python code it shows up as bytes
+    _assert_python_variable_value("test_msg['test_field']", "b'42'");
+
+    LogMessageValueType type;
+    const gchar *value = log_msg_get_value_by_name_with_type(msg, "test_field", NULL, &type);
+
+    // in LogMessage it is INTEGER
+    cr_assert(type == LM_VT_INTEGER);
+    cr_assert_str_eq(value, "42");
 
     Py_XDECREF(msg_object);
   }
@@ -151,7 +280,7 @@ Test(python_log_message, test_python_logmessage_set_value_indirect)
   PyGILState_STATE gstate;
   gstate = PyGILState_Ensure();
   {
-    PyObject *msg_object = py_log_message_new(msg);
+    PyObject *msg_object = py_log_message_new(msg, configuration);
 
     PyDict_SetItemString(_python_main_dict, "test_msg", msg_object);
 
@@ -162,9 +291,7 @@ Test(python_log_message, test_python_logmessage_set_value_indirect)
     const gchar *get_value_indirect = "indirect = test_msg['indirect']";
     PyRun_String(get_value_indirect, Py_file_input, _python_main_dict, _python_main_dict);
 
-    gchar *res_indirect = _dict_clone_value(_python_main_dict, "indirect");
-    cr_assert_str_eq(res_indirect, "test_value");
-    g_free(res_indirect);
+    _assert_python_variable_value("indirect", "b'test_value'");
 
     Py_XDECREF(msg_object);
   }
@@ -177,7 +304,7 @@ Test(python_log_message, test_py_is_log_message)
   gstate = PyGILState_Ensure();
 
   LogMessage *msg = log_msg_new_empty();
-  PyObject *msg_object = py_log_message_new(msg);
+  PyObject *msg_object = py_log_message_new(msg, configuration);
 
   cr_assert(py_is_log_message(msg_object));
   cr_assert_not(py_is_log_message(_python_main));
@@ -238,14 +365,57 @@ Test(python_log_message, test_py_log_message_set_pri)
 
   PyObject *arg = Py_BuildValue("i", pri);
   PyLogMessage *py_msg = _construct_py_log_msg(NULL);
-  Py_DECREF(arg);
 
   PyObject *ret = _py_invoke_method_by_name((PyObject *) py_msg, "set_pri", arg, NULL, NULL);
   Py_XDECREF(ret);
 
   cr_assert_eq(py_msg->msg->pri, pri);
 
+  ret = _py_invoke_method_by_name((PyObject *) py_msg, "get_pri", NULL, NULL, NULL);
+  pri = PyLong_AsLong(ret);
+  Py_XDECREF(ret);
+
+  cr_assert_eq(py_msg->msg->pri, pri);
+  py_msg->msg->pri = 55;
+
+  ret = _py_invoke_method_by_name((PyObject *) py_msg, "get_pri", NULL, NULL, NULL);
+  pri = PyLong_AsLong(ret);
+  Py_XDECREF(ret);
+  cr_assert_eq(py_msg->msg->pri, pri);
+
   Py_DECREF(py_msg);
+  Py_DECREF(arg);
+  PyGILState_Release(gstate);
+}
+
+Test(python_log_message, test_py_log_message_set_timestamp)
+{
+  PyGILState_STATE gstate;
+  gstate = PyGILState_Ensure();
+
+  PyObject *arg = Py_BuildValue("O", py_datetime_from_msec(1664890194123));
+  PyLogMessage *py_msg = _construct_py_log_msg(NULL);
+
+  PyObject *ret = _py_invoke_method_by_name((PyObject *) py_msg, "set_timestamp", arg, NULL, NULL);
+  cr_assert(ret);
+  Py_XDECREF(ret);
+
+  cr_assert_eq(py_msg->msg->timestamps[LM_TS_STAMP].ut_sec, 1664890194);
+  cr_assert_eq(py_msg->msg->timestamps[LM_TS_STAMP].ut_usec, 123000);
+
+  UnixTime ut;
+
+  py_msg->msg->timestamps[LM_TS_STAMP].ut_sec++;
+  ret = _py_invoke_method_by_name((PyObject *) py_msg, "get_timestamp", NULL, NULL, NULL);
+  cr_assert(ret);
+  py_datetime_to_unix_time(ret, &ut);
+
+  cr_assert_eq(ut.ut_sec, 1664890195);
+  cr_assert_eq(ut.ut_usec, 123000);
+  Py_XDECREF(ret);
+
+  Py_DECREF(py_msg);
+  Py_DECREF(arg);
   PyGILState_Release(gstate);
 }
 
@@ -291,7 +461,7 @@ Test(python_log_message, test_python_logmessage_keys)
   log_msg_get_value_handle("unused_value");
 
   PyGILState_STATE gstate = PyGILState_Ensure();
-  PyObject *py_msg = py_log_message_new(msg);
+  PyObject *py_msg = py_log_message_new(msg, configuration);
 
   PyObject *keys = _py_invoke_method_by_name(py_msg, "keys", NULL, "PyLogMessageTest", NULL);
   cr_assert_not_null(keys);
@@ -300,7 +470,8 @@ Test(python_log_message, test_python_logmessage_keys)
   for (Py_ssize_t i = 0; i < PyList_Size(keys); ++i)
     {
       PyObject *py_key = PyList_GetItem(keys, i);
-      const gchar *key = _py_get_string_as_string(py_key);
+      const gchar *key;
+      py_bytes_or_string_to_string(py_key, &key);
       cr_assert_not_null(key);
 
       NVHandle handle = log_msg_get_value_handle(key);
