@@ -48,7 +48,7 @@ static struct
 } handles;
 
 static inline gboolean
-_process_any_char(const guchar **data, gint *left)
+_skip_char(const guchar **data, gint *left)
 {
   if (*left < 1)
     return FALSE;
@@ -59,50 +59,8 @@ _process_any_char(const guchar **data, gint *left)
   return TRUE;
 }
 
-
-static gboolean
-log_msg_parse_pri(LogMessage *self, const guchar **data, gint *length, guint flags, guint16 default_pri)
-{
-  int pri;
-  gboolean success = TRUE;
-  const guchar *src = *data;
-  gint left = *length;
-
-  if (left && src[0] == '<')
-    {
-      _process_any_char(&src, &left);
-      pri = 0;
-      while (left && *src != '>')
-        {
-          if (isdigit(*src))
-            {
-              pri = pri * 10 + ((*src) - '0');
-            }
-          else
-            {
-              return FALSE;
-            }
-          _process_any_char(&src, &left);
-        }
-      self->pri = pri;
-      if (left)
-        {
-          _process_any_char(&src, &left);
-        }
-    }
-  /* No priority info in the buffer? Just assign a default. */
-  else
-    {
-      self->pri = default_pri != 0xFFFF ? default_pri : (EVT_FAC_USER | EVT_PRI_NOTICE);
-    }
-
-  *data = src;
-  *length = left;
-  return success;
-}
-
 static gint
-log_msg_parse_skip_chars(LogMessage *self, const guchar **data, gint *length, const gchar *chars, gint max_len)
+_skip_chars(const guchar **data, gint *length, const gchar *chars, gint max_len)
 {
   const guchar *src = *data;
   gint left = *length;
@@ -110,7 +68,7 @@ log_msg_parse_skip_chars(LogMessage *self, const guchar **data, gint *length, co
 
   while (max_len && left && _strchr_optimized_for_single_char_haystack(chars, *src))
     {
-      _process_any_char(&src, &left);
+      _skip_char(&src, &left);
       num_skipped++;
       if (max_len >= 0)
         max_len--;
@@ -121,14 +79,14 @@ log_msg_parse_skip_chars(LogMessage *self, const guchar **data, gint *length, co
 }
 
 static gboolean
-log_msg_parse_skip_space(LogMessage *self, const guchar **data, gint *length)
+_skip_space(const guchar **data, gint *length)
 {
   const guchar *src = *data;
   gint left = *length;
 
   if (left > 0 && *src == ' ')
     {
-      _process_any_char(&src, &left);
+      _skip_char(&src, &left);
     }
   else
     {
@@ -141,7 +99,7 @@ log_msg_parse_skip_space(LogMessage *self, const guchar **data, gint *length)
 }
 
 static gint
-log_msg_parse_skip_chars_until(LogMessage *self, const guchar **data, gint *length, const gchar *delims)
+_skip_chars_until(const guchar **data, gint *length, const gchar *delims)
 {
   const guchar *src = *data;
   gint left = *length;
@@ -149,7 +107,7 @@ log_msg_parse_skip_chars_until(LogMessage *self, const guchar **data, gint *leng
 
   while (left && _strchr_optimized_for_single_char_haystack(delims, *src) == 0)
     {
-      _process_any_char(&src, &left);
+      _skip_char(&src, &left);
       num_skipped++;
     }
   *data = src;
@@ -157,8 +115,49 @@ log_msg_parse_skip_chars_until(LogMessage *self, const guchar **data, gint *leng
   return num_skipped;
 }
 
+static gboolean
+_syslog_format_parse_pri(LogMessage *msg, const guchar **data, gint *length, guint flags, guint16 default_pri)
+{
+  int pri;
+  gboolean success = TRUE;
+  const guchar *src = *data;
+  gint left = *length;
+
+  if (left && src[0] == '<')
+    {
+      _skip_char(&src, &left);
+      pri = 0;
+      while (left && *src != '>')
+        {
+          if (isdigit(*src))
+            {
+              pri = pri * 10 + ((*src) - '0');
+            }
+          else
+            {
+              return FALSE;
+            }
+          _skip_char(&src, &left);
+        }
+      msg->pri = pri;
+      if (left)
+        {
+          _skip_char(&src, &left);
+        }
+    }
+  /* No priority info in the buffer? Just assign a default. */
+  else
+    {
+      msg->pri = default_pri != 0xFFFF ? default_pri : (EVT_FAC_USER | EVT_PRI_NOTICE);
+    }
+
+  *data = src;
+  *length = left;
+  return success;
+}
+
 static void
-log_msg_parse_column(LogMessage *self, NVHandle handle, const guchar **data, gint *length, gint max_length)
+_syslog_format_parse_column(LogMessage *msg, NVHandle handle, const guchar **data, gint *length, gint max_length)
 {
   const guchar *src, *space;
   gint left;
@@ -181,7 +180,7 @@ log_msg_parse_column(LogMessage *self, NVHandle handle, const guchar **data, gin
       if ((*length - left) > 1 || (*data)[0] != '-')
         {
           gint len = (*length - left) > max_length ? max_length : (*length - left);
-          log_msg_set_value(self, handle, (gchar *) *data, len);
+          log_msg_set_value(msg, handle, (gchar *) *data, len);
         }
     }
   *data = src;
@@ -189,7 +188,7 @@ log_msg_parse_column(LogMessage *self, NVHandle handle, const guchar **data, gin
 }
 
 static void
-log_msg_parse_cisco_sequence_id(LogMessage *self, const guchar **data, gint *length)
+_syslog_format_parse_cisco_sequence_id(LogMessage *msg, const guchar **data, gint *length)
 {
   const guchar *src = *data;
   gint left = *length;
@@ -199,18 +198,18 @@ log_msg_parse_cisco_sequence_id(LogMessage *self, const guchar **data, gint *len
     {
       if (!isdigit(*src))
         return;
-      if (!_process_any_char(&src, &left))
+      if (!_skip_char(&src, &left))
         return;
     }
-  if (!_process_any_char(&src, &left))
+  if (!_skip_char(&src, &left))
     return;
 
   /* if the next char is not space, then we may try to read a date */
 
-  if (*src != ' ')
+  if (!left || *src != ' ')
     return;
 
-  log_msg_set_value(self, handles.cisco_seqid, (gchar *) *data, *length - left - 1);
+  log_msg_set_value(msg, handles.cisco_seqid, (gchar *) *data, *length - left - 1);
 
   *data = src;
   *length = left;
@@ -218,31 +217,35 @@ log_msg_parse_cisco_sequence_id(LogMessage *self, const guchar **data, gint *len
 }
 
 static void
-log_msg_parse_cisco_timestamp_attributes(LogMessage *self, const guchar **data, gint *length, gint parse_flags)
+_syslog_format_parse_cisco_timestamp_attributes(LogMessage *msg, const guchar **data, gint *length, gint parse_flags)
 {
   const guchar *src = *data;
   gint left = *length;
 
+  if (!left)
+    return;
+
   /* Cisco timestamp extensions, the first '*' indicates that the clock is
    * unsynced, '.' if it is known to be synced */
   if (G_UNLIKELY(src[0] == '*'))
     {
       if (!(parse_flags & LP_NO_PARSE_DATE))
-        log_msg_set_value(self, handles.is_synced, "0", 1);
-      _process_any_char(&src, &left);
+        log_msg_set_value(msg, handles.is_synced, "0", 1);
+      _skip_char(&src, &left);
     }
   else if (G_UNLIKELY(src[0] == '.'))
     {
       if (!(parse_flags & LP_NO_PARSE_DATE))
-        log_msg_set_value(self, handles.is_synced, "1", 1);
-      _process_any_char(&src, &left);
+        log_msg_set_value(msg, handles.is_synced, "1", 1);
+      _skip_char(&src, &left);
     }
   *data = src;
   *length = left;
 }
 
 static gboolean
-log_msg_parse_timestamp(UnixTime *stamp, const guchar **data, gint *length, guint parse_flags, glong recv_timezone_ofs)
+_syslog_format_parse_timestamp(UnixTime *stamp, const guchar **data, gint *length, guint parse_flags,
+                               glong recv_timezone_ofs)
 {
   gboolean result;
   WallClockTime wct = WALL_CLOCK_TIME_INIT;
@@ -273,14 +276,15 @@ log_msg_parse_timestamp(UnixTime *stamp, const guchar **data, gint *length, guin
 }
 
 static gboolean
-log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guint parse_flags, glong recv_timezone_ofs)
+_syslog_format_parse_date(LogMessage *msg, const guchar **data, gint *length, guint parse_flags,
+                          glong recv_timezone_ofs)
 {
-  UnixTime *stamp = &self->timestamps[LM_TS_STAMP];
+  UnixTime *stamp = &msg->timestamps[LM_TS_STAMP];
 
   unix_time_unset(stamp);
-  if (!log_msg_parse_timestamp(stamp, data, length, parse_flags, recv_timezone_ofs))
+  if (!_syslog_format_parse_timestamp(stamp, data, length, parse_flags, recv_timezone_ofs))
     {
-      *stamp = self->timestamps[LM_TS_RECVD];
+      *stamp = msg->timestamps[LM_TS_RECVD];
       unix_time_set_timezone(stamp, recv_timezone_ofs);
       return FALSE;
     }
@@ -289,7 +293,7 @@ log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guint pa
 }
 
 static gboolean
-log_msg_parse_version(LogMessage *self, const guchar **data, gint *length)
+_syslog_format_parse_version(LogMessage *msg, const guchar **data, gint *length)
 {
   const guchar *src = *data;
   gint left = *length;
@@ -305,7 +309,7 @@ log_msg_parse_version(LogMessage *self, const guchar **data, gint *length)
         {
           return FALSE;
         }
-      _process_any_char(&src, &left);
+      _skip_char(&src, &left);
     }
   if (version != 1)
     return FALSE;
@@ -316,7 +320,7 @@ log_msg_parse_version(LogMessage *self, const guchar **data, gint *length)
 }
 
 static void
-log_msg_parse_legacy_program_name(LogMessage *self, const guchar **data, gint *length, guint flags)
+_syslog_format_parse_legacy_program_name(LogMessage *msg, const guchar **data, gint *length, guint flags)
 {
   /* the data pointer will not change */
   const guchar *src, *prog_start;
@@ -327,36 +331,36 @@ log_msg_parse_legacy_program_name(LogMessage *self, const guchar **data, gint *l
   prog_start = src;
   while (left && *src != ' ' && *src != '[' && *src != ':')
     {
-      _process_any_char(&src, &left);
+      _skip_char(&src, &left);
     }
-  log_msg_set_value(self, LM_V_PROGRAM, (gchar *) prog_start, src - prog_start);
+  log_msg_set_value(msg, LM_V_PROGRAM, (gchar *) prog_start, src - prog_start);
   if (left > 0 && *src == '[')
     {
       const guchar *pid_start = src + 1;
       while (left && *src != ' ' && *src != ']' && *src != ':')
         {
-          _process_any_char(&src, &left);
+          _skip_char(&src, &left);
         }
       if (left)
         {
-          log_msg_set_value(self, LM_V_PID, (gchar *) pid_start, src - pid_start);
+          log_msg_set_value(msg, LM_V_PID, (gchar *) pid_start, src - pid_start);
         }
       if (left > 0 && *src == ']')
         {
-          _process_any_char(&src, &left);
+          _skip_char(&src, &left);
         }
     }
   if (left > 0 && *src == ':')
     {
-      _process_any_char(&src, &left);
+      _skip_char(&src, &left);
     }
   if (left > 0 && *src == ' ')
     {
-      _process_any_char(&src, &left);
+      _skip_char(&src, &left);
     }
   if ((flags & LP_STORE_LEGACY_MSGHDR))
     {
-      log_msg_set_value(self, LM_V_LEGACY_MSGHDR, (gchar *) *data, *length - left);
+      log_msg_set_value(msg, LM_V_LEGACY_MSGHDR, (gchar *) *data, *length - left);
     }
   *data = src;
   *length = left;
@@ -382,13 +386,19 @@ _init_parse_hostname_invalid_chars(void)
                 i == '.' || i == ':' ||
                 i == '@' || i == '/'))
             {
-              invalid_chars[i >> 8] |= 1 << (i % 8);
+              invalid_chars[i / 8] |= 1 << (i % 8);
             }
         }
       invalid_chars[0] |= 0x1;
     }
 }
 
+static inline gboolean
+_is_invalid_hostname_char(guchar c)
+{
+  return invalid_chars[c / 8] & (1 << (c % 8));
+}
+
 typedef struct _IPv6Heuristics
 {
   gint8 current_segment;
@@ -431,9 +441,9 @@ ipv6_heuristics_feed_gchar(IPv6Heuristics *self, gchar c)
 }
 
 static void
-log_msg_parse_hostname(LogMessage *self, const guchar **data, gint *length,
-                       const guchar **hostname_start, int *hostname_len,
-                       guint flags, regex_t *bad_hostname)
+_syslog_format_parse_hostname(LogMessage *msg, const guchar **data, gint *length,
+                              const guchar **hostname_start, int *hostname_len,
+                              guint flags, regex_t *bad_hostname)
 {
   /* FIXME: support nil value support  with new protocol*/
   const guchar *src, *oldsrc;
@@ -461,12 +471,12 @@ log_msg_parse_hostname(LogMessage *self, const guchar **data, gint *length,
           break;
         }
 
-      if (G_UNLIKELY((flags & LP_CHECK_HOSTNAME) && (invalid_chars[((guint) *src) >> 8] & (1 << (((guint) *src) % 8)))))
+      if (G_UNLIKELY((flags & LP_CHECK_HOSTNAME) && _is_invalid_hostname_char(*src)))
         {
           break;
         }
       hostname_buf[dst++] = *src;
-      _process_any_char(&src, &left);
+      _skip_char(&src, &left);
     }
   hostname_buf[dst] = 0;
 
@@ -498,18 +508,18 @@ log_msg_parse_hostname(LogMessage *self, const guchar **data, gint *length,
 }
 
 /**
- * log_msg_parse:
- * @self: LogMessage instance to store parsed information into
+ * _syslog_format_parse:
+ * @msg: LogMessage instance to store parsed information into
  * @data: message
  * @length: length of the message pointed to by @data
  * @flags: value affecting how the message is parsed (bits from LP_*)
  *
  * Parse an http://www.syslog.cc/ietf/drafts/draft-ietf-syslog-protocol-23.txt formatted log
  * message for structured data elements and store the parsed information
- * in @self.values and dup the SD string. Parsing is affected by the bits set @flags argument.
+ * in @msg.values and dup the SD string. Parsing is affected by the bits set @flags argument.
  **/
-static gboolean
-log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, const MsgFormatOptions *options)
+gboolean
+_syslog_format_parse_sd(LogMessage *msg, const guchar **data, gint *length, const MsgFormatOptions *options)
 {
   /*
    * STRUCTURED-DATA = NILVALUE / 1*SD-ELEMENT
@@ -548,22 +558,22 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, const MsgF
   if (left && src[0] == '-')
     {
       /* Nothing to do here */
-      _process_any_char(&src, &left);
+      _skip_char(&src, &left);
     }
   else if (left && src[0] == '[')
     {
-      _process_any_char(&src, &left);
+      _skip_char(&src, &left);
       open_sd++;
       do
         {
-          if (!isascii(*src) || *src == '=' || *src == ' ' || *src == ']' || *src == '"')
+          if (!left || !isascii(*src) || *src == '=' || *src == ' ' || *src == ']' || *src == '"')
             goto error;
           /* read sd_id */
           pos = 0;
           while (left && *src != ' ' && *src != ']')
             {
-              /* the sd_id_name is max 255, the other chars are only stored in the self->sd_str*/
-              if (pos < sizeof(sd_id_name) - 1 - logmsg_sd_prefix_len)
+              /* the sd_id_name is max 255, the other chars are only stored in the msg->sd_str*/
+              if (pos < sizeof(sd_id_name) - 1 - options->sdata_prefix_len)
                 {
                   if (isascii(*src) && *src != '=' && *src != ' ' && *src != ']' && *src != '"')
                     {
@@ -579,7 +589,7 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, const MsgF
                 {
                   goto error;
                 }
-              _process_any_char(&src, &left);
+              _skip_char(&src, &left);
             }
 
           if (pos == 0)
@@ -587,26 +597,27 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, const MsgF
 
           sd_id_name[pos] = 0;
           sd_id_len = pos;
-          strcpy(sd_value_name, logmsg_sd_prefix);
-          strncpy(sd_value_name + logmsg_sd_prefix_len, sd_id_name, sizeof(sd_value_name) - logmsg_sd_prefix_len);
-          if (*src == ']')
+          strcpy(sd_value_name, options->sdata_prefix);
+          strncpy(sd_value_name + options->sdata_prefix_len, sd_id_name, sizeof(sd_value_name) - options->sdata_prefix_len);
+
+          if (left && *src == ']')
             {
-              log_msg_set_value_by_name(self, sd_value_name, "", 0);
+              log_msg_set_value_by_name(msg, sd_value_name, "", 0);
             }
           else
             {
-              sd_value_name[logmsg_sd_prefix_len + pos] = '.';
+              sd_value_name[options->sdata_prefix_len + pos] = '.';
             }
 
           /* read sd-element */
           while (left && *src != ']')
             {
               if (left && *src == ' ') /* skip the ' ' before the parameter name */
-                _process_any_char(&src, &left);
+                _skip_char(&src, &left);
               else
                 goto error;
 
-              if (!isascii(*src) || *src == '=' || *src == ' ' || *src == ']' || *src == '"')
+              if (!left || !isascii(*src) || *src == '=' || *src == ' ' || *src == ']' || *src == '"')
                 goto error;
 
               /* read sd-param */
@@ -627,14 +638,14 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, const MsgF
                     {
                       goto error;
                     }
-                  _process_any_char(&src, &left);
+                  _skip_char(&src, &left);
                 }
               sd_param_name[pos] = 0;
-              strncpy(&sd_value_name[logmsg_sd_prefix_len + 1 + sd_id_len], sd_param_name,
-                      sizeof(sd_value_name) - logmsg_sd_prefix_len - 1 - sd_id_len);
+              strncpy(&sd_value_name[options->sdata_prefix_len + 1 + sd_id_len], sd_param_name,
+                      sizeof(sd_value_name) - options->sdata_prefix_len - 1 - sd_id_len);
 
               if (left && *src == '=')
-                _process_any_char(&src, &left);
+                _skip_char(&src, &left);
               else
                 goto error;
 
@@ -644,7 +655,7 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, const MsgF
                 {
                   gboolean quote = FALSE;
                   /* opening quote */
-                  _process_any_char(&src, &left);
+                  _skip_char(&src, &left);
                   pos = 0;
 
                   while (left && (*src != '"' || quote))
@@ -662,7 +673,7 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, const MsgF
                             }
                           else if (!quote &&  *src == ']')
                             {
-                              _process_any_char(&src, &left);
+                              _skip_char(&src, &left);
                               goto error;
                             }
                           if (pos < sizeof(sd_param_value) - 1)
@@ -672,27 +683,44 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, const MsgF
                             }
                           quote = FALSE;
                         }
-                      _process_any_char(&src, &left);
+                      _skip_char(&src, &left);
                     }
                   sd_param_value[pos] = 0;
                   sd_param_value_len = pos;
 
                   if (left && *src == '"')/* closing quote */
-                    _process_any_char(&src, &left);
+                    _skip_char(&src, &left);
                   else
                     goto error;
                 }
+              else if (left)
+                {
+                  pos = 0;
+
+                  while (left && (*src != ' ' && *src != ']'))
+                    {
+                      if (pos < sizeof(sd_param_value) - 1)
+                        {
+                          sd_param_value[pos] = *src;
+                          pos++;
+                        }
+                      _skip_char(&src, &left);
+                    }
+                  sd_param_value[pos] = 0;
+                  sd_param_value_len = pos;
+
+                }
               else
                 {
                   goto error;
                 }
 
-              log_msg_set_value_by_name(self, sd_value_name, sd_param_value, sd_param_value_len);
+              log_msg_set_value_by_name(msg, sd_value_name, sd_param_value, sd_param_value_len);
             }
 
           if (left && *src == ']')
             {
-              _process_any_char(&src, &left);
+              _skip_char(&src, &left);
               open_sd--;
             }
           else
@@ -704,12 +732,16 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, const MsgF
           if (left && *src == '[')
             {
               /* new structured data begins, thus continue iteration */
-              _process_any_char(&src, &left);
+              _skip_char(&src, &left);
               open_sd++;
             }
         }
       while (left && open_sd != 0);
     }
+  else
+    {
+      goto error;
+    }
   ret = TRUE;
 error:
   /* FIXME: what happens if an error occurs? there's no way to return a
@@ -722,27 +754,45 @@ error:
   return ret;
 }
 
+gboolean
+_syslog_format_parse_sd_column(LogMessage *msg, const guchar **data, gint *length, const MsgFormatOptions *options)
+{
+  if (*length == 0)
+    return TRUE;
+
+  guchar first_char = (*data)[0];
+  if (first_char == '-' || first_char == '[')
+    return _syslog_format_parse_sd(msg, data, length, options);
+
+  /* the SDATA block is not there, skip parsing it as this is how we have
+   * processed SDATA blocks since we added RFC5424.  This is more forgiving
+   * than strict RFC5424 but apps have a bad history of conforming to it
+   * anyway.  */
+  return TRUE;
+}
+
 static gboolean
-log_msg_parse_legacy_header(LogMessage *self, const guchar **data, gint *length, const MsgFormatOptions *parse_options)
+_syslog_format_parse_legacy_header(LogMessage *msg, const guchar **data, gint *length,
+                                   const MsgFormatOptions *parse_options)
 {
   const guchar *src = *data;
   gint left = *length;
   GTimeVal now;
 
-  log_msg_parse_cisco_sequence_id(self, &src, &left);
-  log_msg_parse_skip_chars(self, &src, &left, " ", -1);
-  log_msg_parse_cisco_timestamp_attributes(self, &src, &left, parse_options->flags);
+  _syslog_format_parse_cisco_sequence_id(msg, &src, &left);
+  _skip_chars(&src, &left, " ", -1);
+  _syslog_format_parse_cisco_timestamp_attributes(msg, &src, &left, parse_options->flags);
 
   cached_g_current_time(&now);
-  if (log_msg_parse_date(self, &src, &left, parse_options->flags & ~LP_SYSLOG_PROTOCOL,
-                         time_zone_info_get_offset(parse_options->recv_time_zone_info, (time_t)now.tv_sec)))
+  if (_syslog_format_parse_date(msg, &src, &left, parse_options->flags & ~LP_SYSLOG_PROTOCOL,
+                                time_zone_info_get_offset(parse_options->recv_time_zone_info, (time_t)now.tv_sec)))
     {
       /* Expected format: hostname program[pid]: */
       /* Possibly: Message forwarded from hostname: ... */
       const guchar *hostname_start = NULL;
       int hostname_len = 0;
 
-      log_msg_parse_skip_chars(self, &src, &left, " ", -1);
+      _skip_chars(&src, &left, " ", -1);
 
       /* Detect funny AIX syslogd forwarded message. */
       if (G_UNLIKELY(left >= (sizeof(aix_fwd_string) - 1) &&
@@ -751,8 +801,8 @@ log_msg_parse_legacy_header(LogMessage *self, const guchar **data, gint *length,
           src += sizeof(aix_fwd_string) - 1;
           left -= sizeof(aix_fwd_string) - 1;
           hostname_start = src;
-          hostname_len = log_msg_parse_skip_chars_until(self, &src, &left, ":");
-          log_msg_parse_skip_chars(self, &src, &left, " :", -1);
+          hostname_len = _skip_chars_until(&src, &left, ":");
+          _skip_chars(&src, &left, " :", -1);
         }
 
       /* Now, try to tell if it's a "last message repeated" line */
@@ -767,21 +817,21 @@ log_msg_parse_legacy_header(LogMessage *self, const guchar **data, gint *length,
             {
               /* Don't parse a hostname if it is local */
               /* It's a regular ol' message. */
-              log_msg_parse_hostname(self, &src, &left, &hostname_start, &hostname_len, parse_options->flags,
-                                     parse_options->bad_hostname);
+              _syslog_format_parse_hostname(msg, &src, &left, &hostname_start, &hostname_len, parse_options->flags,
+                                            parse_options->bad_hostname);
 
               /* Skip whitespace. */
-              log_msg_parse_skip_chars(self, &src, &left, " ", -1);
+              _skip_chars(&src, &left, " ", -1);
             }
 
           /* Try to extract a program name */
-          log_msg_parse_legacy_program_name(self, &src, &left, parse_options->flags);
+          _syslog_format_parse_legacy_program_name(msg, &src, &left, parse_options->flags);
         }
 
       /* If we did manage to find a hostname, store it. */
       if (hostname_start)
         {
-          log_msg_set_value(self, LM_V_HOST, (gchar *) hostname_start, hostname_len);
+          log_msg_set_value(msg, LM_V_HOST, (gchar *) hostname_start, hostname_len);
         }
     }
   else
@@ -790,15 +840,15 @@ log_msg_parse_legacy_header(LogMessage *self, const guchar **data, gint *length,
       /* Different format */
 
       /* A kernel message? Use 'kernel' as the program name. */
-      if (((self->pri & LOG_FACMASK) == LOG_KERN && (parse_options->flags & LP_LOCAL) != 0))
+      if (((msg->pri & LOG_FACMASK) == LOG_KERN && (parse_options->flags & LP_LOCAL) != 0))
         {
-          log_msg_set_value(self, LM_V_PROGRAM, "kernel", 6);
+          log_msg_set_value(msg, LM_V_PROGRAM, "kernel", 6);
         }
       /* No, not a kernel message. */
       else
         {
           /* Capture the program name */
-          log_msg_parse_legacy_program_name(self, &src, &left, parse_options->flags);
+          _syslog_format_parse_legacy_program_name(msg, &src, &left, parse_options->flags);
         }
     }
   *data = src;
@@ -807,19 +857,19 @@ log_msg_parse_legacy_header(LogMessage *self, const guchar **data, gint *length,
 }
 
 /**
- * log_msg_parse_legacy:
- * @self: LogMessage instance to store parsed information into
+ * _syslog_format_parse_legacy:
+ * @msg: LogMessage instance to store parsed information into
  * @data: message
  * @length: length of the message pointed to by @data
  * @flags: value affecting how the message is parsed (bits from LP_*)
  *
  * Parse an RFC3164 formatted log message and store the parsed information
- * in @self. Parsing is affected by the bits set @flags argument.
+ * in @msg. Parsing is affected by the bits set @flags argument.
  **/
 static gboolean
-log_msg_parse_legacy(const MsgFormatOptions *parse_options,
-                     const guchar *data, gint length,
-                     LogMessage *self, gsize *position)
+_syslog_format_parse_legacy(const MsgFormatOptions *parse_options,
+                            const guchar *data, gint length,
+                            LogMessage *msg, gsize *position)
 {
   const guchar *src;
   gint left;
@@ -827,13 +877,13 @@ log_msg_parse_legacy(const MsgFormatOptions *parse_options,
   src = (const guchar *) data;
   left = length;
 
-  if (!log_msg_parse_pri(self, &src, &left, parse_options->flags, parse_options->default_pri))
+  if (!_syslog_format_parse_pri(msg, &src, &left, parse_options->flags, parse_options->default_pri))
     {
       goto error;
     }
 
   if ((parse_options->flags & LP_NO_HEADER) == 0)
-    log_msg_parse_legacy_header(self, &src, &left, parse_options);
+    _syslog_format_parse_legacy_header(msg, &src, &left, parse_options);
 
   if (parse_options->flags & LP_SANITIZE_UTF8 && !g_utf8_validate((gchar *) src, left, NULL))
     {
@@ -849,18 +899,18 @@ log_msg_parse_legacy(const MsgFormatOptions *parse_options,
 
       /* MUST NEVER BE REALLOCATED */
       g_assert(sanitized_message.str == buf);
-      log_msg_set_value(self, LM_V_MESSAGE, sanitized_message.str, sanitized_message.len);
-      self->flags |= LF_UTF8;
+      log_msg_set_value(msg, LM_V_MESSAGE, sanitized_message.str, sanitized_message.len);
+      msg->flags |= LF_UTF8;
     }
   else
     {
-      log_msg_set_value(self, LM_V_MESSAGE, (gchar *) src, left);
+      log_msg_set_value(msg, LM_V_MESSAGE, (gchar *) src, left);
 
       /* we don't need revalidation if sanitize already said it was valid utf8 */
       if ((parse_options->flags & LP_VALIDATE_UTF8) &&
           ((parse_options->flags & LP_SANITIZE_UTF8) == 0) &&
           g_utf8_validate((gchar *) src, left, NULL))
-        self->flags |= LF_UTF8;
+        msg->flags |= LF_UTF8;
     }
 
   return TRUE;
@@ -870,13 +920,14 @@ error:
 }
 
 /**
- * log_msg_parse_syslog_proto:
+ * _syslog_format_parse_syslog_proto:
  *
  * Parse a message according to the latest syslog-protocol drafts.
  **/
 static gboolean
-log_msg_parse_syslog_proto(const MsgFormatOptions *parse_options, const guchar *data, gint length, LogMessage *self,
-                           gsize *position)
+_syslog_format_parse_syslog_proto(const MsgFormatOptions *parse_options, const guchar *data, gint length,
+                                  LogMessage *msg,
+                                  gsize *position)
 {
   /**
    *  SYSLOG-MSG      = HEADER SP STRUCTURED-DATA [SP MSG]
@@ -897,28 +948,30 @@ log_msg_parse_syslog_proto(const MsgFormatOptions *parse_options, const guchar *
   left = length;
 
 
-  if (!log_msg_parse_pri(self, &src, &left, parse_options->flags, parse_options->default_pri) ||
-      !log_msg_parse_version(self, &src, &left))
+  if (!_syslog_format_parse_pri(msg, &src, &left, parse_options->flags, parse_options->default_pri) ||
+      !_syslog_format_parse_version(msg, &src, &left))
     {
-      return log_msg_parse_legacy(parse_options, data, length, self, position);
+      if ((parse_options->flags & LP_NO_RFC3164_FALLBACK) == 0)
+        return _syslog_format_parse_legacy(parse_options, data, length, msg, position);
+      return FALSE;
     }
 
-  if (!log_msg_parse_skip_space(self, &src, &left))
+  if (!_skip_space(&src, &left))
     {
       goto error;
     }
 
   /* ISO time format */
-  if (!log_msg_parse_date(self, &src, &left, parse_options->flags,
-                          time_zone_info_get_offset(parse_options->recv_time_zone_info, time(NULL))))
+  if (!_syslog_format_parse_date(msg, &src, &left, parse_options->flags,
+                                 time_zone_info_get_offset(parse_options->recv_time_zone_info, time(NULL))))
     goto error;
 
-  if (!log_msg_parse_skip_space(self, &src, &left))
+  if (!_skip_space(&src, &left))
     goto error;
 
   /* hostname 255 ascii */
-  log_msg_parse_hostname(self, &src, &left, &hostname_start, &hostname_len, parse_options->flags, NULL);
-  if (!log_msg_parse_skip_space(self, &src, &left))
+  _syslog_format_parse_hostname(msg, &src, &left, &hostname_start, &hostname_len, parse_options->flags, NULL);
+  if (!_skip_space(&src, &left))
     {
       src++;
       goto error;
@@ -928,33 +981,33 @@ log_msg_parse_syslog_proto(const MsgFormatOptions *parse_options, const guchar *
     ;
   else if (hostname_start)
     {
-      log_msg_set_value(self, LM_V_HOST, (gchar *) hostname_start, hostname_len);
+      log_msg_set_value(msg, LM_V_HOST, (gchar *) hostname_start, hostname_len);
     }
 
   /* application name 48 ascii*/
-  log_msg_parse_column(self, LM_V_PROGRAM, &src, &left, 48);
-  if (!log_msg_parse_skip_space(self, &src, &left))
+  _syslog_format_parse_column(msg, LM_V_PROGRAM, &src, &left, 48);
+  if (!_skip_space(&src, &left))
     goto error;
 
   /* process id 128 ascii */
-  log_msg_parse_column(self, LM_V_PID, &src, &left, 128);
-  if (!log_msg_parse_skip_space(self, &src, &left))
+  _syslog_format_parse_column(msg, LM_V_PID, &src, &left, 128);
+  if (!_skip_space(&src, &left))
     goto error;
 
   /* message id 32 ascii */
-  log_msg_parse_column(self, LM_V_MSGID, &src, &left, 32);
-  if (!log_msg_parse_skip_space(self, &src, &left))
+  _syslog_format_parse_column(msg, LM_V_MSGID, &src, &left, 32);
+  if (!_skip_space(&src, &left))
     goto error;
 
   /* structured data part */
-  if (!log_msg_parse_sd(self, &src, &left, parse_options))
+  if (!_syslog_format_parse_sd_column(msg, &src, &left, parse_options))
     goto error;
 
   /* checking if there are remaining data in log message */
   if (left != 0)
     {
       /* optional part of the log message [SP MSG] */
-      if (!log_msg_parse_skip_space(self, &src, &left))
+      if (!_skip_space(&src, &left))
         {
           goto error;
         }
@@ -962,16 +1015,16 @@ log_msg_parse_syslog_proto(const MsgFormatOptions *parse_options, const guchar *
       if (left >= 3 && memcmp(src, "\xEF\xBB\xBF", 3) == 0)
         {
           /* we have a BOM, this is UTF8 */
-          self->flags |= LF_UTF8;
+          msg->flags |= LF_UTF8;
           src += 3;
           left -= 3;
         }
       else if ((parse_options->flags & LP_VALIDATE_UTF8) && g_utf8_validate((gchar *) src, left, NULL))
         {
-          self->flags |= LF_UTF8;
+          msg->flags |= LF_UTF8;
         }
     }
-  log_msg_set_value(self, LM_V_MESSAGE, (gchar *) src, left);
+  log_msg_set_value(msg, LM_V_MESSAGE, (gchar *) src, left);
   return TRUE;
 error:
   *position = src - data;
@@ -991,9 +1044,9 @@ syslog_format_handler(const MsgFormatOptions *parse_options,
 
   msg->initial_parse = TRUE;
   if (parse_options->flags & LP_SYSLOG_PROTOCOL)
-    success = log_msg_parse_syslog_proto(parse_options, data, length, msg, problem_position);
+    success = _syslog_format_parse_syslog_proto(parse_options, data, length, msg, problem_position);
   else
-    success = log_msg_parse_legacy(parse_options, data, length, msg, problem_position);
+    success = _syslog_format_parse_legacy(parse_options, data, length, msg, problem_position);
   msg->initial_parse = FALSE;
 
   return success;
