@@ -26,10 +26,12 @@
 
 #include "logsource.h"
 #include "journald-subsystem.h"
+#include "journald-helper.h"
 
 typedef struct _JournalReader JournalReader;
 
-typedef struct _JournalReaderOptions {
+typedef struct _JournalReaderOptions
+{
   LogSourceOptions super;
   gboolean initialized;
   gint fetch_limit;
@@ -39,13 +41,25 @@ typedef struct _JournalReaderOptions {
   TimeZoneInfo *recv_time_zone_info;
   gchar *prefix;
   guint32 max_field_size;
+  gchar *namespace;
+  GList *matches;
+  gboolean match_boot;
 } JournalReaderOptions;
 
-JournalReader *journal_reader_new(GlobalConfig *cfg, Journald *journal);
-void journal_reader_set_persist_name(JournalReader *self, gchar *persist_name);
-void journal_reader_set_options(LogPipe *s, LogPipe *control, JournalReaderOptions *options, gint stats_level, gint stats_source, const gchar *stats_id, const gchar *stats_instance);
+JournalReader *journal_reader_new(GlobalConfig *cfg);
+void journal_reader_set_options(LogPipe *s, LogPipe *control, JournalReaderOptions *options, const gchar *stats_id,
+                                const gchar *stats_instance);
 
 void journal_reader_options_init(JournalReaderOptions *options, GlobalConfig *cfg, const gchar *group_name);
+void journal_reader_options_set_default_severity(JournalReaderOptions *self, gint severity);
+void journal_reader_options_set_default_facility(JournalReaderOptions *self, gint facility);
+void journal_reader_options_set_time_zone(JournalReaderOptions *self, gchar *time_zone);
+void journal_reader_options_set_prefix(JournalReaderOptions *self, gchar *prefix);
+void journal_reader_options_set_max_field_size(JournalReaderOptions *self, gint max_field_size);
+void journal_reader_options_set_namespace(JournalReaderOptions *self, gchar *namespace);
+void journal_reader_options_set_log_fetch_limit(JournalReaderOptions *self, gint log_fetch_limit);
+void journal_reader_options_set_matches(JournalReaderOptions *self, GList *matches);
+void journal_reader_options_set_match_boot(JournalReaderOptions *self, gboolean enable);
 void journal_reader_options_defaults(JournalReaderOptions *options);
 void journal_reader_options_destroy(JournalReaderOptions *options);
 
