@@ -117,6 +117,12 @@ extern int systemd_journal_debug;
     KW_BATCH_LINES = 10087,        /* KW_BATCH_LINES  */
     KW_BATCH_TIMEOUT = 10088,      /* KW_BATCH_TIMEOUT  */
     KW_TRIM_LARGE_MESSAGES = 10089, /* KW_TRIM_LARGE_MESSAGES  */
+    KW_STATS = 10400,              /* KW_STATS  */
+    KW_FREQ = 10401,               /* KW_FREQ  */
+    KW_LEVEL = 10402,              /* KW_LEVEL  */
+    KW_LIFETIME = 10403,           /* KW_LIFETIME  */
+    KW_MAX_DYNAMIC = 10404,        /* KW_MAX_DYNAMIC  */
+    KW_SYSLOG_STATS = 10405,       /* KW_SYSLOG_STATS  */
     KW_CHAIN_HOSTNAMES = 10090,    /* KW_CHAIN_HOSTNAMES  */
     KW_NORMALIZE_HOSTNAMES = 10091, /* KW_NORMALIZE_HOSTNAMES  */
     KW_KEEP_HOSTNAME = 10092,      /* KW_KEEP_HOSTNAME  */
@@ -146,10 +152,10 @@ extern int systemd_journal_debug;
     KW_HOST_OVERRIDE = 10166,      /* KW_HOST_OVERRIDE  */
     KW_THROTTLE = 10170,           /* KW_THROTTLE  */
     KW_THREADED = 10171,           /* KW_THREADED  */
-    KW_PASS_UNIX_CREDENTIALS = 10231, /* KW_PASS_UNIX_CREDENTIALS  */
-    KW_PERSIST_NAME = 10302,       /* KW_PERSIST_NAME  */
-    KW_READ_OLD_RECORDS = 10304,   /* KW_READ_OLD_RECORDS  */
-    KW_USE_SYSLOGNG_PID = 10305,   /* KW_USE_SYSLOGNG_PID  */
+    KW_PASS_UNIX_CREDENTIALS = 10180, /* KW_PASS_UNIX_CREDENTIALS  */
+    KW_PERSIST_NAME = 10181,       /* KW_PERSIST_NAME  */
+    KW_READ_OLD_RECORDS = 10182,   /* KW_READ_OLD_RECORDS  */
+    KW_USE_SYSLOGNG_PID = 10183,   /* KW_USE_SYSLOGNG_PID  */
     KW_FLAGS = 10190,              /* KW_FLAGS  */
     KW_PAD_SIZE = 10200,           /* KW_PAD_SIZE  */
     KW_TIME_ZONE = 10201,          /* KW_TIME_ZONE  */
@@ -175,6 +181,7 @@ extern int systemd_journal_debug;
     KW_TEMPLATE_FUNCTION = 10272,  /* KW_TEMPLATE_FUNCTION  */
     KW_DEFAULT_FACILITY = 10300,   /* KW_DEFAULT_FACILITY  */
     KW_DEFAULT_SEVERITY = 10301,   /* KW_DEFAULT_SEVERITY  */
+    KW_SDATA_PREFIX = 10302,       /* KW_SDATA_PREFIX  */
     KW_PORT = 10323,               /* KW_PORT  */
     KW_USE_TIME_RECVD = 10340,     /* KW_USE_TIME_RECVD  */
     KW_FACILITY = 10350,           /* KW_FACILITY  */
@@ -190,6 +197,7 @@ extern int systemd_journal_debug;
     KW_VALUE = 10372,              /* KW_VALUE  */
     KW_YES = 10380,                /* KW_YES  */
     KW_NO = 10381,                 /* KW_NO  */
+    KW_AUTO = 10382,               /* KW_AUTO  */
     KW_IFDEF = 10410,              /* KW_IFDEF  */
     KW_ENDIF = 10411,              /* KW_ENDIF  */
     LL_DOTDOT = 10420,             /* LL_DOTDOT  */
@@ -222,7 +230,9 @@ extern int systemd_journal_debug;
     KW_SYSTEMD_JOURNAL = 10524,    /* KW_SYSTEMD_JOURNAL  */
     KW_PREFIX = 10525,             /* KW_PREFIX  */
     KW_MAX_FIELD_SIZE = 10526,     /* KW_MAX_FIELD_SIZE  */
-    KW_NAMESPACE = 10527           /* KW_NAMESPACE  */
+    KW_NAMESPACE = 10527,          /* KW_NAMESPACE  */
+    KW_MATCHES = 10528,            /* KW_MATCHES  */
+    KW_MATCH_BOOT = 10529          /* KW_MATCH_BOOT  */
   };
   typedef enum systemd_journal_tokentype systemd_journal_token_kind_t;
 #endif
@@ -286,6 +296,12 @@ extern int systemd_journal_debug;
 #define KW_BATCH_LINES 10087
 #define KW_BATCH_TIMEOUT 10088
 #define KW_TRIM_LARGE_MESSAGES 10089
+#define KW_STATS 10400
+#define KW_FREQ 10401
+#define KW_LEVEL 10402
+#define KW_LIFETIME 10403
+#define KW_MAX_DYNAMIC 10404
+#define KW_SYSLOG_STATS 10405
 #define KW_CHAIN_HOSTNAMES 10090
 #define KW_NORMALIZE_HOSTNAMES 10091
 #define KW_KEEP_HOSTNAME 10092
@@ -315,10 +331,10 @@ extern int systemd_journal_debug;
 #define KW_HOST_OVERRIDE 10166
 #define KW_THROTTLE 10170
 #define KW_THREADED 10171
-#define KW_PASS_UNIX_CREDENTIALS 10231
-#define KW_PERSIST_NAME 10302
-#define KW_READ_OLD_RECORDS 10304
-#define KW_USE_SYSLOGNG_PID 10305
+#define KW_PASS_UNIX_CREDENTIALS 10180
+#define KW_PERSIST_NAME 10181
+#define KW_READ_OLD_RECORDS 10182
+#define KW_USE_SYSLOGNG_PID 10183
 #define KW_FLAGS 10190
 #define KW_PAD_SIZE 10200
 #define KW_TIME_ZONE 10201
@@ -344,6 +360,7 @@ extern int systemd_journal_debug;
 #define KW_TEMPLATE_FUNCTION 10272
 #define KW_DEFAULT_FACILITY 10300
 #define KW_DEFAULT_SEVERITY 10301
+#define KW_SDATA_PREFIX 10302
 #define KW_PORT 10323
 #define KW_USE_TIME_RECVD 10340
 #define KW_FACILITY 10350
@@ -359,6 +376,7 @@ extern int systemd_journal_debug;
 #define KW_VALUE 10372
 #define KW_YES 10380
 #define KW_NO 10381
+#define KW_AUTO 10382
 #define KW_IFDEF 10410
 #define KW_ENDIF 10411
 #define LL_DOTDOT 10420
@@ -392,6 +410,8 @@ extern int systemd_journal_debug;
 #define KW_PREFIX 10525
 #define KW_MAX_FIELD_SIZE 10526
 #define KW_NAMESPACE 10527
+#define KW_MATCHES 10528
+#define KW_MATCH_BOOT 10529
 
 /* Value type.  */
 #if ! defined SYSTEMD_JOURNAL_STYPE && ! defined SYSTEMD_JOURNAL_STYPE_IS_DECLARED
