@@ -20,16 +20,17 @@
  * COPYING for details.
  *
  */
+#include <criterion/criterion.h>
+
 #include "test-source.h"
 #include "syslog-ng.h"
-#include "testutils.h"
 #include "stats/stats.h"
 
-struct _TestSource {
+struct _TestSource
+{
   LogPipe super;
   JournalReaderOptions options;
   JournalReader *reader;
-  Journald *journald_mock;
   GList *tests;
   GList *current_test;
   TestCase *current_test_case;
@@ -43,16 +44,18 @@ static gboolean
 __init(LogPipe *s)
 {
   TestSource *self = (TestSource *)s;
-  self->reader = journal_reader_new(configuration, self->journald_mock);
+  GlobalConfig *cfg = log_pipe_get_config(s);
+
+  self->reader = journal_reader_new(cfg);
   journal_reader_options_defaults(&self->options);
   if (self->current_test_case && self->current_test_case->init)
     {
-      self->current_test_case->init(self->current_test_case, self, self->journald_mock, self->reader, &self->options);
+      self->current_test_case->init(self->current_test_case, self, self->reader, &self->options);
     }
-  journal_reader_options_init(&self->options, configuration, "test");
-  journal_reader_set_options((LogPipe *)self->reader, &self->super, &self->options, 3, SCS_JOURNALD, "test", "1");
+  journal_reader_options_init(&self->options, cfg, "test");
+  journal_reader_set_options((LogPipe *)self->reader, &self->super, &self->options, "test", "1");
   log_pipe_append((LogPipe *)self->reader, &self->super);
-  assert_true(log_pipe_init((LogPipe *)self->reader), ASSERTION_ERROR("Can't initialize reader"));
+  cr_assert(log_pipe_init((LogPipe *)self->reader), "%s", "Can't initialize reader");
   return TRUE;
 }
 
@@ -70,12 +73,11 @@ static void
 __free(LogPipe *s)
 {
   TestSource *self = (TestSource *)s;
-  journald_free(self->journald_mock);
   g_list_free(self->tests);
 }
 
 static void
-__queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options, gpointer user_data)
+__queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options)
 {
   TestSource *self = (TestSource *)s;
   if (self->current_test_case && self->current_test_case->checker)
@@ -113,7 +115,6 @@ test_source_new(GlobalConfig *cfg)
   self->super.deinit = __deinit;
   self->super.free_fn = __free;
   self->super.queue = __queue;
-  self->journald_mock = journald_mock_new();
   journal_reader_options_defaults(&self->options);
 
   IV_TASK_INIT(&self->start);
